@@ -1,28 +1,30 @@
+import tempfile
 import numpy as np
-
-from desc.backend import jnp
-from desc.boundary_conditions import LCFSConstraint, PoincareConstraint
-from desc.utils import sign, copy_coeffs
+import warnings
+import numbers
+from desc.backend import jnp, sign, put
+from desc.utils import copy_coeffs
 from desc.grid import Grid, LinearGrid
 from desc.basis import DoubleFourierSeries, ZernikePolynomial
 from desc.transform import Transform
-from .core import Surface, cart2polvec, pol2cartvec
+from desc.io import InputReader
+from .core import Surface
+from .utils import xyz2rpz_vec, rpz2xyz_vec, xyz2rpz, rpz2xyz
 
 __all__ = ["FourierRZToroidalSurface", "ZernikeRZToroidalSection"]
 
 
 class FourierRZToroidalSurface(Surface):
-    """Toroidal surface represented by a double fourier series in poloidal angle
-    theta and toroidal angle phi/zeta
+    """Toroidal surface represented by Fourier series in poloidal and toroidal angles.
 
     Parameters
     ----------
-    R_mn, Z_mn : array-like, shape(k,)
+    R_lmn, Z_lmn : array-like, shape(k,)
         Fourier coefficients for R and Z in cylindrical coordinates
     modes_R : array-like, shape(k,2)
-        poloidal and toroidal mode numbers [m,n] for R_n.
+        poloidal and toroidal mode numbers [m,n] for R_lmn.
     modes_Z : array-like, shape(k,2)
-        mode numbers associated with Z_n, defaults to modes_R
+        mode numbers associated with Z_lmn, defaults to modes_R
     NFP : int
         number of field periods
     sym : bool
@@ -38,8 +40,8 @@ class FourierRZToroidalSurface(Surface):
     """
 
     _io_attrs_ = Surface._io_attrs_ + [
-        "_R_mn",
-        "_Z_mn",
+        "_R_lmn",
+        "_Z_lmn",
         "_R_basis",
         "_Z_basis",
         "_R_transform",
@@ -50,8 +52,8 @@ class FourierRZToroidalSurface(Surface):
 
     def __init__(
         self,
-        R_mn=None,
-        Z_mn=None,
+        R_lmn=None,
+        Z_lmn=None,
         modes_R=None,
         modes_Z=None,
         NFP=1,
@@ -61,15 +63,17 @@ class FourierRZToroidalSurface(Surface):
         name="",
     ):
 
-        if R_mn is None:
-            R_mn = np.array([10, 1])
+        if R_lmn is None:
+            R_lmn = np.array([10, 1])
             modes_R = np.array([[0, 0], [1, 0]])
-        if Z_mn is None:
-            Z_mn = np.array([0, 1])
+        if Z_lmn is None:
+            Z_lmn = np.array([0, 1])
             modes_Z = np.array([[0, 0], [-1, 0]])
         if modes_Z is None:
             modes_Z = modes_R
-        modes_R, modes_Z = np.asarray(modes_R), np.asarray(modes_Z)
+        R_lmn, Z_lmn, modes_R, modes_Z = map(
+            np.asarray, (R_lmn, Z_lmn, modes_R, modes_Z)
+        )
 
         assert issubclass(modes_R.dtype.type, np.integer)
         assert issubclass(modes_Z.dtype.type, np.integer)
@@ -78,11 +82,14 @@ class FourierRZToroidalSurface(Surface):
         NR = np.max(abs(modes_R[:, 1]))
         MZ = np.max(abs(modes_Z[:, 0]))
         NZ = np.max(abs(modes_Z[:, 1]))
+        self._L = 0
+        self._M = max(MR, MZ)
+        self._N = max(NR, NZ)
         if sym == "auto":
             if np.all(
-                R_mn[np.where(sign(modes_R[:, 0]) != sign(modes_R[:, 1]))] == 0
+                R_lmn[np.where(sign(modes_R[:, 0]) != sign(modes_R[:, 1]))] == 0
             ) and np.all(
-                Z_mn[np.where(sign(modes_Z[:, 0]) == sign(modes_Z[:, 1]))] == 0
+                Z_lmn[np.where(sign(modes_Z[:, 0]) == sign(modes_Z[:, 1]))] == 0
             ):
                 sym = True
             else:
@@ -95,16 +102,16 @@ class FourierRZToroidalSurface(Surface):
             M=MZ, N=NZ, NFP=NFP, sym="sin" if sym else False
         )
 
-        self._R_mn = copy_coeffs(R_mn, modes_R, self.R_basis.modes[:, 1:])
-        self._Z_mn = copy_coeffs(Z_mn, modes_Z, self.Z_basis.modes[:, 1:])
+        self._R_lmn = copy_coeffs(R_lmn, modes_R, self.R_basis.modes[:, 1:])
+        self._Z_lmn = copy_coeffs(Z_lmn, modes_Z, self.Z_basis.modes[:, 1:])
         self._NFP = NFP
         self._sym = sym
         self.rho = rho
         if grid is None:
             grid = LinearGrid(
                 rho=self.rho,
-                M=2 * max(MR, MZ) + 1,
-                N=2 * max(NR, NZ) + 1,
+                M=4 * self.M + 1,
+                N=4 * self.N + 1,
                 endpoint=True,
             )
         self._grid = grid
@@ -113,22 +120,29 @@ class FourierRZToroidalSurface(Surface):
 
     @property
     def NFP(self):
-        """number of toroidal field periods"""
+        """Number of (toroidal) field periods (int)."""
         return self._NFP
 
+    @NFP.setter
+    def NFP(self, new):
+        assert (
+            isinstance(new, numbers.Real) and int(new) == new and new > 0
+        ), f"NFP should be a positive integer, got {type(new)}"
+        self.change_resolution(NFP=new)
+
     @property
     def R_basis(self):
-        """Spectral basis for R double fourier series"""
+        """Spectral basis for R double Fourier series."""
         return self._R_basis
 
     @property
     def Z_basis(self):
-        """Spectral basis for Z double fourier series"""
+        """Spectral basis for Z double Fourier series."""
         return self._Z_basis
 
     @property
     def grid(self):
-        """Default grid for computation"""
+        """Grid for computation."""
         return self._grid
 
     @grid.setter
@@ -144,46 +158,72 @@ class FourierRZToroidalSurface(Surface):
         self._R_transform.grid = self.grid
         self._Z_transform.grid = self.grid
 
-    def change_resolution(self, M, N):
+    def change_resolution(self, *args, **kwargs):
         """Change the maximum poloidal and toroidal resolution"""
-        R_modes_old = self.R_basis.modes
-        Z_modes_old = self.Z_basis.modes
-        self.R_basis.change_resolution(M=M, N=N)
-        self.Z_basis.change_resolution(M=M, N=N)
-        self._R_transform, self._Z_transform = self._get_transforms(self.grid)
-        self.R_mn = copy_coeffs(self.R_mn, R_modes_old, self.R_basis.modes)
-        self.Z_mn = copy_coeffs(self.Z_mn, Z_modes_old, self.Z_basis.modes)
+        assert ((len(args) in [2, 3]) and (len(kwargs) == 0 or "NFP" in kwargs)) or (
+            len(args) == 0
+        ), "change_resolution should be called with 2 or 3 positional arguments or only keyword arguments"
+        L = kwargs.get("L", None)
+        M = kwargs.get("M", None)
+        N = kwargs.get("N", None)
+        NFP = kwargs.get("NFP", None)
+        self._NFP = NFP if NFP is not None else self.NFP
+        if L is not None:
+            warnings.warn(
+                "FourierRZToroidalSurface does not have a radial resolution, ignoring L"
+            )
+        if len(args) == 2:
+            M, N = args
+        elif len(args) == 3:
+            L, M, N = args
+
+        if (
+            ((N is not None) and (N != self.N))
+            or ((M is not None) and (M != self.M))
+            or (NFP is not None)
+        ):
+            M = M if M is not None else self.M
+            N = N if N is not None else self.N
+            R_modes_old = self.R_basis.modes
+            Z_modes_old = self.Z_basis.modes
+            self.R_basis.change_resolution(M=M, N=N, NFP=self.NFP)
+            self.Z_basis.change_resolution(M=M, N=N, NFP=self.NFP)
+            self._R_transform, self._Z_transform = self._get_transforms(self.grid)
+            self.R_lmn = copy_coeffs(self.R_lmn, R_modes_old, self.R_basis.modes)
+            self.Z_lmn = copy_coeffs(self.Z_lmn, Z_modes_old, self.Z_basis.modes)
+            self._M = M
+            self._N = N
 
     @property
-    def R_mn(self):
-        """Spectral coefficients for R"""
-        return self._R_mn
+    def R_lmn(self):
+        """Spectral coefficients for R."""
+        return self._R_lmn
 
-    @R_mn.setter
-    def R_mn(self, new):
+    @R_lmn.setter
+    def R_lmn(self, new):
         if len(new) == self.R_basis.num_modes:
-            self._R_mn = jnp.asarray(new)
+            self._R_lmn = jnp.asarray(new)
         else:
             raise ValueError(
-                f"R_mn should have the same size as the basis, got {len(new)} for basis with {self.R_basis.num_modes} modes"
+                f"R_lmn should have the same size as the basis, got {len(new)} for basis with {self.R_basis.num_modes} modes"
             )
 
     @property
-    def Z_mn(self):
-        """Spectral coefficients for Z"""
-        return self._Z_mn
+    def Z_lmn(self):
+        """Spectral coefficients for Z."""
+        return self._Z_lmn
 
-    @Z_mn.setter
-    def Z_mn(self, new):
+    @Z_lmn.setter
+    def Z_lmn(self, new):
         if len(new) == self.Z_basis.num_modes:
-            self._Z_mn = jnp.asarray(new)
+            self._Z_lmn = jnp.asarray(new)
         else:
             raise ValueError(
-                f"Z_mn should have the same size as the basis, got {len(new)} for basis with {self.Z_basis.num_modes} modes"
+                f"Z_lmn should have the same size as the basis, got {len(new)} for basis with {self.Z_basis.num_modes} modes"
             )
 
     def get_coeffs(self, m, n=0):
-        """Get fourier coefficients for given mode number(s)"""
+        """Get Fourier coefficients for given mode number(s)."""
         n = np.atleast_1d(n).astype(int)
         m = np.atleast_1d(m).astype(int)
 
@@ -199,12 +239,12 @@ class FourierRZToroidalSurface(Surface):
             (mn[:, np.newaxis, :] == self.Z_basis.modes[np.newaxis, :, 1:]).all(axis=-1)
         )
 
-        R[idxR[0]] = self.R_mn[idxR[1]]
-        Z[idxZ[0]] = self.Z_mn[idxZ[1]]
+        R[idxR[0]] = self.R_lmn[idxR[1]]
+        Z[idxZ[0]] = self.Z_lmn[idxZ[1]]
         return R, Z
 
     def set_coeffs(self, m, n=0, R=None, Z=None):
-        """set specific fourier coefficients"""
+        """Set specific Fourier coefficients."""
         m, n, R, Z = (
             np.atleast_1d(m),
             np.atleast_1d(n),
@@ -213,12 +253,12 @@ class FourierRZToroidalSurface(Surface):
         )
         m, n, R, Z = np.broadcast_arrays(m, n, R, Z)
         for mm, nn, RR, ZZ in zip(m, n, R, Z):
-            idxR = self.R_basis.get_idx(0, mm, nn)
-            idxZ = self.Z_basis.get_idx(0, mm, nn)
             if RR is not None:
-                self.R_mn[idxR] = RR
+                idxR = self.R_basis.get_idx(0, mm, nn)
+                self.R_lmn = put(self.R_lmn, idxR, RR)
             if ZZ is not None:
-                self.Z_mn[idxZ] = ZZ
+                idxZ = self.Z_basis.get_idx(0, mm, nn)
+                self.Z_lmn = put(self.Z_lmn, idxZ, ZZ)
 
     def _get_transforms(self, grid=None):
         if grid is None:
@@ -237,92 +277,104 @@ class FourierRZToroidalSurface(Surface):
             grid,
             self.R_basis,
             derivs=np.array(
-                [[0, 0, 0], [0, 0, 1], [0, 0, 2], [0, 1, 0], [0, 0, 2], [0, 1, 1]]
+                [[0, 0, 0], [0, 1, 0], [0, 2, 0], [0, 0, 1], [0, 0, 2], [0, 1, 1]]
             ),
         )
         Z_transform = Transform(
             grid,
             self.Z_basis,
             derivs=np.array(
-                [[0, 0, 0], [0, 0, 1], [0, 0, 2], [0, 1, 0], [0, 0, 2], [0, 1, 1]]
+                [[0, 0, 0], [0, 1, 0], [0, 2, 0], [0, 0, 1], [0, 0, 2], [0, 1, 1]]
             ),
         )
         return R_transform, Z_transform
 
     def compute_curvature(self, params=None, grid=None):
-        """Compute gaussian and mean curvature"""
+        """Compute gaussian and mean curvature."""
         raise NotImplementedError()
 
-    def compute_coordinates(self, R_mn=None, Z_mn=None, grid=None, dt=0, dz=0):
-        """Compute values using specified coefficients
+    def compute_coordinates(
+        self, R_lmn=None, Z_lmn=None, grid=None, dt=0, dz=0, basis="rpz"
+    ):
+        """Compute values using specified coefficients.
 
         Parameters
         ----------
-        R_mn, Z_mn: array-like
-            fourier coefficients for R, Z. Defaults to self.R_mn, self.Z_mn
+        R_lmn, Z_lmn: array-like
+            fourier coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,2pi)
         dt, dz: int
             derivative order to compute in theta, zeta
+        basis : {"rpz", "xyz"}
+            coordinate system for returned points
 
         Returns
         -------
         values : ndarray, shape(k,3)
-            R, phi, Z coordinates of the surface at points specified in grid
+            R, phi, Z or x, y, z coordinates of the surface at points specified in grid
         """
-        if R_mn is None:
-            R_mn = self.R_mn
-        if Z_mn is None:
-            Z_mn = self.Z_mn
+        assert basis.lower() in ["rpz", "xyz"]
+        if R_lmn is None:
+            R_lmn = self.R_lmn
+        if Z_lmn is None:
+            Z_lmn = self.Z_lmn
         R_transform, Z_transform = self._get_transforms(grid)
 
         if dz == 0:
-            R = R_transform.transform(R_mn, dt=dt, dz=0)
-            Z = Z_transform.transform(Z_mn, dt=dt, dz=0)
+            R = R_transform.transform(R_lmn, dt=dt, dz=0)
+            Z = Z_transform.transform(Z_lmn, dt=dt, dz=0)
             phi = R_transform.grid.nodes[:, 2] * (dt == 0)
-            return jnp.stack([R, phi, Z], axis=1)
-        if dz == 1:
-            R0 = R_transform.transform(R_mn, dt=dt, dz=0)
-            dR = R_transform.transform(R_mn, dt=dt, dz=1)
-            dZ = Z_transform.transform(Z_mn, dt=dt, dz=1)
+            coords = jnp.stack([R, phi, Z], axis=1)
+        elif dz == 1:
+            R0 = R_transform.transform(R_lmn, dt=dt, dz=0)
+            dR = R_transform.transform(R_lmn, dt=dt, dz=1)
+            dZ = Z_transform.transform(Z_lmn, dt=dt, dz=1)
             dphi = R0 * (dt == 0)
-            return jnp.stack([dR, dphi, dZ], axis=1)
-        if dz == 2:
-            R0 = R_transform.transform(R_mn, dt=dt, dz=0)
-            dR = R_transform.transform(R_mn, dt=dt, dz=1)
-            d2R = R_transform.transform(R_mn, dt=dt, dz=2)
-            d2Z = Z_transform.transform(Z_mn, dt=dt, dz=2)
+            coords = jnp.stack([dR, dphi, dZ], axis=1)
+        elif dz == 2:
+            R0 = R_transform.transform(R_lmn, dt=dt, dz=0)
+            dR = R_transform.transform(R_lmn, dt=dt, dz=1)
+            d2R = R_transform.transform(R_lmn, dt=dt, dz=2)
+            d2Z = Z_transform.transform(Z_lmn, dt=dt, dz=2)
             R = d2R - R0
             Z = d2Z
             # 2nd derivative wrt to phi = 0
             phi = 2 * dR * (dt == 0)
-            return jnp.stack([R, phi, Z], axis=1)
-        if dz == 3:
-            R0 = R_transform.transform(R_mn, dt=dt, dz=0)
-            dR = R_transform.transform(R_mn, dt=dt, dz=1)
-            d2R = R_transform.transform(R_mn, dt=dt, dz=2)
-            d3R = R_transform.transform(R_mn, dt=dt, dz=3)
-            d3Z = Z_transform.transform(Z_mn, dt=dt, dz=3)
+            coords = jnp.stack([R, phi, Z], axis=1)
+        elif dz == 3:
+            R0 = R_transform.transform(R_lmn, dt=dt, dz=0)
+            dR = R_transform.transform(R_lmn, dt=dt, dz=1)
+            d2R = R_transform.transform(R_lmn, dt=dt, dz=2)
+            d3R = R_transform.transform(R_lmn, dt=dt, dz=3)
+            d3Z = Z_transform.transform(Z_lmn, dt=dt, dz=3)
             R = d3R - 3 * dR
             Z = d3Z
             phi = (3 * d2R - R0) * (dt == 0)
-            return jnp.stack([R, phi, Z], axis=1)
-        raise NotImplementedError(
-            "Derivatives higher than 3 have not been implemented in cylindrical coordinates"
-        )
+            coords = jnp.stack([R, phi, Z], axis=1)
+        else:
+            raise NotImplementedError(
+                "Derivatives higher than 3 have not been implemented in cylindrical coordinates"
+            )
+        if basis.lower() == "xyz":
+            if (dt > 0) or (dz > 0):
+                coords = rpz2xyz_vec(coords, phi=R_transform.grid.nodes[:, 2])
+            else:
+                coords = rpz2xyz(coords)
+        return coords
 
-    def compute_normal(self, R_mn=None, Z_mn=None, grid=None, coords="rpz"):
-        """Compute normal vector to surface on default grid
+    def compute_normal(self, R_lmn=None, Z_lmn=None, grid=None, basis="rpz"):
+        """Compute normal vector to surface on default grid.
 
         Parameters
         ----------
-        R_mn, Z_mn: array-like
-            fourier coefficients for R, Z. Defaults to self.R_mn, self.Z_mn
+        R_lmn, Z_lmn: array-like
+            fourier coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,2pi)
-        coords : {"rpz", "xyz"}
+        basis : {"rpz", "xyz"}
             basis vectors to use for normal vector representation
 
         Returns
@@ -330,31 +382,31 @@ class FourierRZToroidalSurface(Surface):
         N : ndarray, shape(k,3)
             normal vector to surface in specified coordinates
         """
-        assert coords.lower() in ["rpz", "xyz"]
+        assert basis.lower() in ["rpz", "xyz"]
 
-        if R_mn is None:
-            R_mn = self.R_mn
-        if Z_mn is None:
-            Z_mn = self.Z_mn
+        if R_lmn is None:
+            R_lmn = self.R_lmn
+        if Z_lmn is None:
+            Z_lmn = self.Z_lmn
         R_transform, Z_transform = self._get_transforms(grid)
 
-        r_t = self.compute_coordinates(R_mn, Z_mn, grid, dt=1)
-        r_z = self.compute_coordinates(R_mn, Z_mn, grid, dz=1)
+        r_t = self.compute_coordinates(R_lmn, Z_lmn, grid, dt=1)
+        r_z = self.compute_coordinates(R_lmn, Z_lmn, grid, dz=1)
 
         N = jnp.cross(r_t, r_z, axis=1)
         N = N / jnp.linalg.norm(N, axis=1)[:, jnp.newaxis]
-        if coords.lower() == "xyz":
+        if basis.lower() == "xyz":
             phi = R_transform.grid.nodes[:, 2]
-            N = pol2cartvec(N, phi=phi)
+            N = rpz2xyz_vec(N, phi=phi)
         return N
 
-    def compute_surface_area(self, R_mn=None, Z_mn=None, grid=None):
-        """Compute surface area via quadrature
+    def compute_surface_area(self, R_lmn=None, Z_lmn=None, grid=None):
+        """Compute surface area via quadrature.
 
         Parameters
         ----------
-        R_mn, Z_mn: array-like
-            fourier coefficients for R, Z. Defaults to self.R_mn, self.Z_mn
+        R_lmn, Z_lmn: array-like
+            fourier coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,2pi)
@@ -364,36 +416,66 @@ class FourierRZToroidalSurface(Surface):
         area : float
             surface area
         """
-        if R_mn is None:
-            R_mn = self.R_mn
-        if Z_mn is None:
-            Z_mn = self.Z_mn
+        if R_lmn is None:
+            R_lmn = self.R_lmn
+        if Z_lmn is None:
+            Z_lmn = self.Z_lmn
         R_transform, Z_transform = self._get_transforms(grid)
 
-        r_t = self.compute_coordinates(R_mn, Z_mn, grid, dt=1)
-        r_z = self.compute_coordinates(R_mn, Z_mn, grid, dz=1)
+        r_t = self.compute_coordinates(R_lmn, Z_lmn, grid, dt=1)
+        r_z = self.compute_coordinates(R_lmn, Z_lmn, grid, dz=1)
 
         N = jnp.cross(r_t, r_z, axis=1)
         return jnp.sum(R_transform.grid.weights * jnp.linalg.norm(N, axis=1))
 
-    def get_constraint(self, R_basis, Z_basis, L_basis):
-        """Get the linear constraint to enforce this surface as a boundary condition"""
-        return LCFSConstraint(
-            R_basis, Z_basis, L_basis, self.R_basis, self.Z_basis, self.R_mn, self.Z_mn
+    @classmethod
+    def from_input_file(cls, path):
+        """Create a surface objective from Fourier coefficients in a DESC or VMEC input file
+
+        Parameters
+        ----------
+        path : Path-like or str
+            path to DESC or VMEC input file
+
+        Returns
+        -------
+        surface : FourierRZToroidalSurface
+            surface with given Fourier coefficients
+
+        """
+        f = open(path, "r")
+        if "&INDATA" in f.readlines()[0]:  # vmec input, convert to desc
+            inputs = InputReader.parse_vmec_inputs(f)
+        else:
+            inputs = InputReader().parse_inputs(f)[-1]
+        if inputs["bdry_ratio"] != 1:
+            warnings.warn(
+                "boundary_ratio = {} != 1, surface may not be as expected".format(
+                    inputs["bdry_ratio"]
+                )
+            )
+        surf = cls(
+            inputs["surface"][:, 3],
+            inputs["surface"][:, 4],
+            inputs["surface"][:, 1:3].astype(int),
+            inputs["surface"][:, 1:3].astype(int),
+            inputs["NFP"],
+            inputs["sym"],
         )
+        return surf
 
 
 class ZernikeRZToroidalSection(Surface):
-    """A toroidal cross section represented by a zernike polynomial in R,Z
+    """A toroidal cross section represented by a Zernike polynomial in R,Z.
 
     Parameters
     ----------
-    R_lm, Z_lm : array-like, shape(k,)
+    R_lmn, Z_lmn : array-like, shape(k,)
         zernike coefficients
     modes_R : array-like, shape(k,2)
-        radial and poloidal mode numbers [l,m] for R_lm
+        radial and poloidal mode numbers [l,m] for R_lmn
     modes_Z : array-like, shape(k,2)
-        radial and poloidal mode numbers [l,m] for Z_lm. If None defaults to modes_R.
+        radial and poloidal mode numbers [l,m] for Z_lmn. If None defaults to modes_R.
     sym : bool
         whether to enforce stellarator symmetry. Default is "auto" which enforces if
         modes are symmetric. If True, non-symmetric modes will be truncated.
@@ -424,8 +506,8 @@ class ZernikeRZToroidalSection(Surface):
     """
 
     _io_attrs_ = Surface._io_attrs_ + [
-        "_R_lm",
-        "_Z_lm",
+        "_R_lmn",
+        "_Z_lmn",
         "_R_basis",
         "_Z_basis",
         "_R_transform",
@@ -436,8 +518,8 @@ class ZernikeRZToroidalSection(Surface):
 
     def __init__(
         self,
-        R_lm=None,
-        Z_lm=None,
+        R_lmn=None,
+        Z_lmn=None,
         modes_R=None,
         modes_Z=None,
         spectral_indexing="fringe",
@@ -446,15 +528,17 @@ class ZernikeRZToroidalSection(Surface):
         grid=None,
         name="",
     ):
-        if R_lm is None:
-            R_lm = np.array([10, 1])
+        if R_lmn is None:
+            R_lmn = np.array([10, 1])
             modes_R = np.array([[0, 0], [1, 1]])
-        if Z_lm is None:
-            Z_lm = np.array([0, 1])
+        if Z_lmn is None:
+            Z_lmn = np.array([0, 1])
             modes_Z = np.array([[0, 0], [1, -1]])
         if modes_Z is None:
             modes_Z = modes_R
-        modes_R, modes_Z = np.asarray(modes_R), np.asarray(modes_Z)
+        R_lmn, Z_lmn, modes_R, modes_Z = map(
+            np.asarray, (R_lmn, Z_lmn, modes_R, modes_Z)
+        )
 
         assert issubclass(modes_R.dtype.type, np.integer)
         assert issubclass(modes_Z.dtype.type, np.integer)
@@ -463,12 +547,15 @@ class ZernikeRZToroidalSection(Surface):
         MR = np.max(abs(modes_R[:, 1]))
         LZ = np.max(abs(modes_Z[:, 0]))
         MZ = np.max(abs(modes_Z[:, 1]))
+        self._L = max(LR, LZ)
+        self._M = max(MR, MZ)
+        self._N = 0
 
         if sym == "auto":
             if np.all(
-                R_lm[np.where(sign(modes_R[:, 0]) != sign(modes_R[:, 1]))] == 0
+                R_lmn[np.where(sign(modes_R[:, 0]) != sign(modes_R[:, 1]))] == 0
             ) and np.all(
-                Z_lm[np.where(sign(modes_Z[:, 0]) == sign(modes_Z[:, 1]))] == 0
+                Z_lmn[np.where(sign(modes_Z[:, 0]) == sign(modes_Z[:, 1]))] == 0
             ):
                 sym = True
             else:
@@ -487,15 +574,15 @@ class ZernikeRZToroidalSection(Surface):
             sym="sin" if sym else False,
         )
 
-        self._R_lm = copy_coeffs(R_lm, modes_R, self.R_basis.modes[:, :2])
-        self._Z_lm = copy_coeffs(Z_lm, modes_Z, self.Z_basis.modes[:, :2])
+        self._R_lmn = copy_coeffs(R_lmn, modes_R, self.R_basis.modes[:, :2])
+        self._Z_lmn = copy_coeffs(Z_lmn, modes_Z, self.Z_basis.modes[:, :2])
         self._sym = sym
         self._spectral_indexing = spectral_indexing
 
         self.zeta = zeta
         if grid is None:
             grid = LinearGrid(
-                L=max(LR, LZ), M=2 * max(MR, MZ) + 1, zeta=self.zeta, endpoint=True
+                L=2 * self.L, M=4 * self.M + 1, zeta=self.zeta, endpoint=True
             )
         self._grid = grid
         self._R_transform, self._Z_transform = self._get_transforms(grid)
@@ -507,17 +594,17 @@ class ZernikeRZToroidalSection(Surface):
 
     @property
     def R_basis(self):
-        """Spectral basis for R zernike polynomial"""
+        """Spectral basis for R Zernike polynomial."""
         return self._R_basis
 
     @property
     def Z_basis(self):
-        """Spectral basis for Z zernike polynomial"""
+        """Spectral basis for Z Zernike polynomial."""
         return self._Z_basis
 
     @property
     def grid(self):
-        """Default grid for computation"""
+        """Grid for computation."""
         return self._grid
 
     @grid.setter
@@ -533,46 +620,65 @@ class ZernikeRZToroidalSection(Surface):
         self._R_transform.grid = self.grid
         self._Z_transform.grid = self.grid
 
-    def change_resolution(self, L, M):
+    def change_resolution(self, *args, **kwargs):
         """Change the maximum radial and poloidal resolution"""
-        R_modes_old = self.R_basis.modes
-        Z_modes_old = self.Z_basis.modes
-        self.R_basis.change_resolution(L=L, M=M)
-        self.Z_basis.change_resolution(L=L, M=M)
-        self._R_transform, self._Z_transform = self._get_transforms(self.grid)
-        self.R_lm = copy_coeffs(self.R_lm, R_modes_old, self.R_basis.modes)
-        self.Z_lm = copy_coeffs(self.Z_lm, Z_modes_old, self.Z_basis.modes)
+        assert ((len(args) in [2, 3]) and len(kwargs) == 0) or (
+            len(args) == 0
+        ), "change_resolution should be called with 2 or 3 positional arguments or only keyword arguments"
+        L = kwargs.get("L", None)
+        M = kwargs.get("M", None)
+        N = kwargs.get("N", None)
+        if N is not None:
+            warnings.warn(
+                "ZernikeRZToroidalSection does not have a toroidal resolution, ignoring N"
+            )
+        if len(args) == 2:
+            L, M = args
+        elif len(args) == 3:
+            L, M, N = args
+
+        if ((L is not None) and (L != self.L)) or ((M is not None) and (M != self.M)):
+            R_modes_old = self.R_basis.modes
+            Z_modes_old = self.Z_basis.modes
+            self.R_basis.change_resolution(L=L, M=M)
+            self.Z_basis.change_resolution(L=L, M=M)
+            self._R_transform, self._Z_transform = self._get_transforms(self.grid)
+            self.R_lmn = copy_coeffs(self.R_lmn, R_modes_old, self.R_basis.modes)
+            self.Z_lmn = copy_coeffs(self.Z_lmn, Z_modes_old, self.Z_basis.modes)
+            self._L = L
+            self._M = M
 
     @property
-    def R_lm(self):
-        """Spectral coefficients for R"""
-        return self._R_lm
+    def R_lmn(self):
+        """Spectral coefficients for R."""
+        return self._R_lmn
 
-    @R_lm.setter
-    def R_lm(self, new):
+    @R_lmn.setter
+    def R_lmn(self, new):
         if len(new) == self.R_basis.num_modes:
-            self._R_lm = jnp.asarray(new)
+            self._R_lmn = jnp.asarray(new)
         else:
             raise ValueError(
-                f"R_lm should have the same size as the basis, got {len(new)} for basis with {self.R_basis.num_modes} modes"
+                f"R_lmn should have the same size as the basis, got {len(new)} for basis with {self.R_basis.num_modes} modes"
             )
 
     @property
-    def Z_lm(self):
-        """Spectral coefficients for Z"""
-        return self._Z_lm
+    def Z_lmn(self):
+        """Spectral coefficients for Z."""
+        return self._Z_lmn
 
-    @Z_lm.setter
-    def Z_lm(self, new):
+    @Z_lmn.setter
+    def Z_lmn(self, new):
         if len(new) == self.Z_basis.num_modes:
-            self._Z_lm = jnp.asarray(new)
+            self._Z_lmn = jnp.asarray(new)
         else:
             raise ValueError(
-                f"Z_lm should have the same size as the basis, got {len(new)} for basis with {self.Z_basis.num_modes} modes"
+                f"Z_lmn should have the same size as the basis, got {len(new)} for basis with {self.Z_basis.num_modes} modes"
             )
 
     def get_coeffs(self, l, m=0):
-        """Get fourier coefficients for given mode number(s)"""
+        """Get Zernike coefficients for given mode number(s)."""
+
         l = np.atleast_1d(l).astype(int)
         m = np.atleast_1d(m).astype(int)
 
@@ -588,12 +694,13 @@ class ZernikeRZToroidalSection(Surface):
             (lm[:, np.newaxis, :] == self.Z_basis.modes[np.newaxis, :, :2]).all(axis=-1)
         )
 
-        R[idxR[0]] = self.R_lm[idxR[1]]
-        Z[idxZ[0]] = self.Z_lm[idxZ[1]]
+        R[idxR[0]] = self.R_lmn[idxR[1]]
+        Z[idxZ[0]] = self.Z_lmn[idxZ[1]]
         return R, Z
 
     def set_coeffs(self, l, m=0, R=None, Z=None):
-        """set specific fourier coefficients"""
+        """Set specific Zernike coefficients."""
+
         l, m, R, Z = (
             np.atleast_1d(l),
             np.atleast_1d(m),
@@ -602,12 +709,12 @@ class ZernikeRZToroidalSection(Surface):
         )
         l, m, R, Z = np.broadcast_arrays(l, m, R, Z)
         for ll, mm, RR, ZZ in zip(l, m, R, Z):
-            idxR = self.R_basis.get_idx(ll, mm, 0)
-            idxZ = self.Z_basis.get_idx(ll, mm, 0)
             if RR is not None:
-                self.R_lm[idxR] = RR
+                idxR = self.R_basis.get_idx(ll, mm, 0)
+                self.R_lmn = put(self.R_lmn, idxR, RR)
             if ZZ is not None:
-                self.Z_lm[idxZ] = ZZ
+                idxZ = self.Z_basis.get_idx(ll, mm, 0)
+                self.Z_lmn = put(self.Z_lmn, idxZ, ZZ)
 
     def _get_transforms(self, grid=None):
         if grid is None:
@@ -639,50 +746,60 @@ class ZernikeRZToroidalSection(Surface):
         return R_transform, Z_transform
 
     def compute_curvature(self, params=None, grid=None):
-        """Compute gaussian and mean curvature"""
+        """Compute gaussian and mean curvature."""
         raise NotImplementedError()
 
-    def compute_coordinates(self, R_lm=None, Z_lm=None, grid=None, dr=0, dt=0):
-        """Compute values using specified coefficients
+    def compute_coordinates(
+        self, R_lmn=None, Z_lmn=None, grid=None, dr=0, dt=0, basis="rpz"
+    ):
+        """Compute values using specified coefficients.
 
         Parameters
         ----------
-        R_lm, Z_lm: array-like
-            zernike coefficients for R, Z. Defaults to self.R_lm, self.Z_lm
+        R_lmn, Z_lmn: array-like
+            zernike coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,1)x(0,2pi)
         dr, dt: int
             derivative order to compute in rho, theta
+        basis : {"rpz", "xyz"}
+            coordinate system for returned points
 
         Returns
         -------
         values : ndarray, shape(k,3)
-            R, phi, Z coordinates of the surface at points specified in grid
+            R, phi, Z or x, y, z coordinates of the surface at points specified in grid
         """
-        if R_lm is None:
-            R_lm = self.R_lm
-        if Z_lm is None:
-            Z_lm = self.Z_lm
+        assert basis.lower() in ["rpz", "xyz"]
+        if R_lmn is None:
+            R_lmn = self.R_lmn
+        if Z_lmn is None:
+            Z_lmn = self.Z_lmn
         R_transform, Z_transform = self._get_transforms(grid)
 
-        R = R_transform.transform(R_lm, dr=dr, dt=dt)
-        Z = Z_transform.transform(Z_lm, dr=dr, dt=dt)
+        R = R_transform.transform(R_lmn, dr=dr, dt=dt)
+        Z = Z_transform.transform(Z_lmn, dr=dr, dt=dt)
         phi = R_transform.grid.nodes[:, 2] * (dr == 0) * (dt == 0)
+        coords = jnp.stack([R, phi, Z], axis=1)
+        if basis.lower() == "xyz":
+            if (dt > 0) or (dr > 0):
+                coords = rpz2xyz_vec(coords, phi=R_transform.grid.nodes[:, 2])
+            else:
+                coords = rpz2xyz(coords)
+        return coords
 
-        return jnp.stack([R, phi, Z], axis=1)
-
-    def compute_normal(self, R_lm=None, Z_lm=None, grid=None, coords="rpz"):
-        """Compute normal vector to surface on default grid
+    def compute_normal(self, R_lmn=None, Z_lmn=None, grid=None, basis="rpz"):
+        """Compute normal vector to surface on default grid.
 
         Parameters
         ----------
-        R_lm, Z_lm: array-like
-            zernike coefficients for R, Z. Defaults to self.R_lm, self.Z_lm
+        R_lmn, Z_lmn: array-like
+            zernike coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,1)x(0,2pi)
-        coords : {"rpz", "xyz"}
+        basis : {"rpz", "xyz"}
             basis vectors to use for normal vector representation
 
         Returns
@@ -690,7 +807,7 @@ class ZernikeRZToroidalSection(Surface):
         N : ndarray, shape(k,3)
             normal vector to surface in specified coordinates
         """
-        assert coords.lower() in ["rpz", "xyz"]
+        assert basis.lower() in ["rpz", "xyz"]
 
         R_transform, Z_transform = self._get_transforms(grid)
 
@@ -699,18 +816,18 @@ class ZernikeRZToroidalSection(Surface):
         # normal vector is a constant 1*phihat
         N = jnp.array([jnp.zeros_like(phi), jnp.ones_like(phi), jnp.zeros_like(phi)]).T
 
-        if coords.lower() == "xyz":
-            N = pol2cartvec(N, phi=phi)
+        if basis.lower() == "xyz":
+            N = rpz2xyz_vec(N, phi=phi)
 
         return N
 
-    def compute_surface_area(self, R_lm=None, Z_lm=None, grid=None):
-        """Compute surface area via quadrature
+    def compute_surface_area(self, R_lmn=None, Z_lmn=None, grid=None):
+        """Compute surface area via quadrature.
 
         Parameters
         ----------
-        R_lm, Z_lm: array-like
-            zernike coefficients for R, Z. Defaults to self.R_lm, self.Z_lm
+        R_lmn, Z_lmn: array-like
+            zernike coefficients for R, Z. Defaults to self.R_lmn, self.Z_lmn
         grid : Grid or array-like
             toroidal coordinates to compute at. Defaults to self.grid
             If an integer, assumes that many linearly spaced points in (0,1)x(0,2pi)
@@ -720,22 +837,16 @@ class ZernikeRZToroidalSection(Surface):
         area : float
             surface area
         """
-        if R_lm is None:
-            R_lm = self.R_lm
-        if Z_lm is None:
-            Z_lm = self.Z_lm
+        if R_lmn is None:
+            R_lmn = self.R_lmn
+        if Z_lmn is None:
+            Z_lmn = self.Z_lmn
         R_transform, Z_transform = self._get_transforms(grid)
 
-        r_r = self.compute_coordinates(R_lm, Z_lm, grid, dr=1)
-        r_t = self.compute_coordinates(R_lm, Z_lm, grid, dt=1)
+        r_r = self.compute_coordinates(R_lmn, Z_lmn, grid, dr=1)
+        r_t = self.compute_coordinates(R_lmn, Z_lmn, grid, dt=1)
 
         N = jnp.cross(r_r, r_t, axis=1)
         return jnp.sum(R_transform.grid.weights * jnp.linalg.norm(N, axis=1)) / (
             2 * np.pi
         )
-
-    def get_constraint(self, R_basis, Z_basis, L_basis):
-        """Get the linear constraint to enforce this surface as a boundary condition"""
-        return PoincareConstraint(
-            R_basis, Z_basis, L_basis, self.R_basis, self.Z_basis, self.R_lm, self.Z_lm
-        )
