@@ -1,5 +1,4 @@
 import numpy as np
-from termcolor import colored
 
 from desc.io import IOAble
 from scipy import special
@@ -8,12 +7,11 @@ __all__ = ["Grid", "LinearGrid", "QuadratureGrid", "ConcentricGrid"]
 
 
 class Grid(IOAble):
-    """Base class for collocation grids
+    """Base class for collocation grids.
 
     Unlike subclasses LinearGrid and ConcentricGrid, the base Grid allows the user
     to pass in a custom set of collocation nodes.
 
-
     Parameters
     ----------
     nodes : ndarray of float, size(num_nodes,3)
@@ -31,13 +29,18 @@ class Grid(IOAble):
         "_NFP",
         "_sym",
         "_nodes",
+        "_spacing",
         "_weights",
         "_axis",
         "_node_pattern",
+        "_num_rho",
+        "_num_theta",
+        "_num_zeta",
     ]
 
     def __init__(self, nodes, sort=True):
 
+        nodes = np.atleast_2d(nodes)
         self._L = np.unique(nodes[:, 0]).size
         self._M = np.unique(nodes[:, 1]).size
         self._N = np.unique(nodes[:, 2]).size
@@ -45,118 +48,123 @@ class Grid(IOAble):
         self._sym = False
         self._node_pattern = "custom"
 
-        self._nodes, self._weights = self._create_nodes(nodes)
+        self._nodes, self._spacing = self._create_nodes(nodes)
 
         self._enforce_symmetry()
         if sort:
             self._sort_nodes()
         self._find_axis()
+        self._count_nodes()
         self._scale_weights()
 
     def _enforce_symmetry(self):
-        """Enforces stellarator symmetry"""
+        """Enforce stellarator symmetry."""
         if self.sym:  # remove nodes with theta > pi
             non_sym_idx = np.where(self.nodes[:, 1] > np.pi)
             self._nodes = np.delete(self.nodes, non_sym_idx, axis=0)
-            self._weights = np.delete(self.weights, non_sym_idx, axis=0)
+            self._spacing = np.delete(self.spacing, non_sym_idx, axis=0)
 
     def _sort_nodes(self):
-        """Sorts nodes for use with FFT"""
-
+        """Sort nodes for use with FFT."""
         sort_idx = np.lexsort((self.nodes[:, 1], self.nodes[:, 0], self.nodes[:, 2]))
         self._nodes = self.nodes[sort_idx]
-        self._weights = self.weights[sort_idx]
+        self._spacing = self.spacing[sort_idx]
 
     def _find_axis(self):
-        """Finds indices of axis nodes"""
+        """Find indices of axis nodes."""
         self._axis = np.where(self.nodes[:, 0] == 0)[0]
 
-    def _scale_weights(self):
-        """Scales weights to sum to full volume and reduces weights for duplicated nodes"""
+    def _count_nodes(self):
+        """Count unique values of coordinates."""
+        self._num_rho = np.unique(self.nodes[:, 0]).size
+        self._num_theta = np.unique(self.nodes[:, 1]).size
+        self._num_zeta = np.unique(self.nodes[:, 2]).size
 
+    def _scale_weights(self):
+        """Scale weights sum to full volume and reduce weights for duplicated nodes."""
         nodes = self.nodes.copy().astype(float)
         nodes[:, 1] %= 2 * np.pi
         nodes[:, 2] %= 2 * np.pi / self.NFP
+        # reduce weights for duplicated nodes
         _, inverse, counts = np.unique(
             nodes, axis=0, return_inverse=True, return_counts=True
         )
-        self._weights /= counts[inverse]
-        self._weights *= 4 * np.pi ** 2 / self._weights.sum()
+        self._spacing /= np.tile(np.atleast_2d(counts[inverse]).T, 3) ** (1 / 3)
+        self._spacing *= (4 * np.pi ** 2 / self.spacing.prod(axis=1).sum()) ** (1 / 3)
+        self._weights = self.spacing.prod(axis=1)
 
     def _create_nodes(self, nodes):
-        """Allows for custom node creation
+        """Allow for custom node creation.
 
         Parameters
         ----------
         nodes : ndarray of float, size(num_nodes,3)
-            node coordinates, in (rho,theta,zeta)
+            Node coordinates, in (rho,theta,zeta).
 
         Returns
         -------
         nodes : ndarray of float, size(num_nodes,3)
-            node coordinates, in (rho,theta,zeta)
+            Node coordinates, in (rho,theta,zeta).
+        spacing : ndarray of float, size(num_nodes,3)
+            Node spacing, in (rho,theta,zeta).
 
         """
         nodes = np.atleast_2d(nodes).reshape((-1, 3))
-        # make weights sum to 4pi^2
-        weights = np.ones(nodes.shape[0]) / nodes.shape[0] * 4 * np.pi ** 2
+        spacing = (  # make weights sum to 4pi^2
+            np.ones_like(nodes) * np.array([1, 2 * np.pi, 2 * np.pi]) / nodes.shape[0]
+        )
         self._L = len(np.unique(nodes[:, 0]))
         self._M = len(np.unique(nodes[:, 1]))
         self._N = len(np.unique(nodes[:, 2]))
-        return nodes, weights
+        return nodes, spacing
 
     @property
     def L(self):
-        """int: radial grid resolution"""
-        if not hasattr(self, "_L"):
-            self._L = 0
-        return self._L
+        """int: Radial grid resolution."""
+        return self.__dict__.setdefault("_L", 0)
 
     @property
     def M(self):
-        """ int: poloidal grid resolution"""
-        if not hasattr(self, "_M"):
-            self._M = 0
-        return self._M
+        """int: Poloidal grid resolution."""
+        return self.__dict__.setdefault("_M", 0)
 
     @property
     def N(self):
-        """ int: toroidal grid resolution"""
-        if not hasattr(self, "_N"):
-            self._N = 0
-        return self._N
+        """int: Toroidal grid resolution."""
+        return self.__dict__.setdefault("_N", 0)
 
     @property
     def NFP(self):
-        """ int: number of field periods"""
-        if not hasattr(self, "_NFP"):
-            self._NFP = 1
-        return self._NFP
+        """int: Number of (toroidal) field periods."""
+        return self.__dict__.setdefault("_NFP", 1)
 
     @property
     def sym(self):
-        """ bool: True for stellarator symmetry, False otherwise"""
-        if not hasattr(self, "_sym"):
-            self._sym = False
-        return self._sym
+        """bool: True for stellarator symmetry, False otherwise."""
+        return self.__dict__.setdefault("_sym", False)
 
     @property
     def nodes(self):
-        """ndarray: node coordinates, in (rho,theta,zeta)"""
-        if not hasattr(self, "_nodes"):
-            self._nodes = np.array([]).reshape((0, 3))
-        return self._nodes
+        """ndarray: Node coordinates, in (rho,theta,zeta)."""
+        return self.__dict__.setdefault("_nodes", np.array([]).reshape((0, 3)))
 
     @nodes.setter
     def nodes(self, nodes):
         self._nodes = nodes
 
+    @property
+    def spacing(self):
+        """ndarray: node spacing, in (rho,theta,zeta)"""
+        return self.__dict__.setdefault("_spacing", np.array([]).reshape((0, 3)))
+
+    @spacing.setter
+    def spacing(self, spacing):
+        self._spacing = spacing
+
     @property
     def weights(self):
-        """ndarray: weight for each node, either exact quadrature or volume based"""
-        if not hasattr(self, "_weights"):
-            self._weights = np.array([]).reshape((0, 3))
-        return self._weights
+        """ndarray: Weight for each node, either exact quadrature or volume based."""
+        return self.__dict__.setdefault("_weights", np.array([]).reshape((0, 3)))
 
     @weights.setter
     def weights(self, weights):
@@ -164,25 +172,36 @@ class Grid(IOAble):
 
     @property
     def num_nodes(self):
-        """int: total number of nodes"""
+        """int: total number of nodes."""
         return self.nodes.shape[0]
 
+    @property
+    def num_rho(self):
+        """int: number of unique rho coordinates"""
+        return self._num_rho
+
+    @property
+    def num_theta(self):
+        """int: number of unique theta coordinates"""
+        return self._num_theta
+
+    @property
+    def num_zeta(self):
+        """int: number of unique zeta coordinates"""
+        return self._num_zeta
+
     @property
     def axis(self):
-        """ndarray: indices of nodes at magnetic axis"""
-        if not hasattr(self, "_axis"):
-            self._axis = np.array([])
-        return self._axis
+        """ndarray: Indices of nodes at magnetic axis."""
+        return self.__dict__.setdefault("_axis", np.array([]))
 
     @property
     def node_pattern(self):
-        """str: pattern for placement of nodes in rho,theta,zeta"""
-        if not hasattr(self, "_node_pattern"):
-            self._node_pattern = None
-        return self._node_pattern
+        """str: Pattern for placement of nodes in (rho,theta,zeta)."""
+        return self.__dict__.setdefault("_node_pattern", "custom")
 
     def __repr__(self):
-        """string form of the object"""
+        """str: string form of the object."""
         return (
             type(self).__name__
             + " at "
@@ -248,7 +267,7 @@ class LinearGrid(Grid):
         self._endpoint = endpoint
         self._node_pattern = "linear"
 
-        self._nodes, self._weights = self._create_nodes(
+        self._nodes, self._spacing = self._create_nodes(
             L=self.L,
             M=self.M,
             N=self.N,
@@ -263,6 +282,7 @@ class LinearGrid(Grid):
         self._enforce_symmetry()
         self._sort_nodes()
         self._find_axis()
+        self._count_nodes()
         self._scale_weights()
 
     def _create_nodes(
@@ -277,7 +297,7 @@ class LinearGrid(Grid):
         theta=None,
         zeta=None,
     ):
-        """
+        """Create grid nodes and weights.
 
         Parameters
         ----------
@@ -305,8 +325,8 @@ class LinearGrid(Grid):
         -------
         nodes : ndarray of float, size(num_nodes,3)
             node coordinates, in (rho,theta,zeta)
-        weights : ndarray of float, size(num_nodes,)
-            weight for each node, based on local volume around the node
+        spacing : ndarray of float, size(num_nodes,3)
+            node spacing, based on local volume around the node
 
         """
         self._L = L
@@ -317,19 +337,22 @@ class LinearGrid(Grid):
         # rho
         if rho is not None:
             r = np.atleast_1d(rho)
-            r0 = r[0]
             self._L = r.size
+        elif self.L == 0:
+            r = np.array([], dtype=float)
         elif self.L == 1:
             r = np.array([1.0])
-            r0 = 0
         else:
             if axis:
                 r0 = 0
             else:
                 r0 = 1.0 / self.L
             r = np.linspace(r0, 1, self.L)
-        dr = (1 - r0) / self.L
-        if dr == 0:
+        if self.L > 0:
+            dr = (r[-1] - r[0]) / self.L
+            if dr == 0:
+                dr = 1
+        else:
             dr = 1
 
         # theta/vartheta
@@ -338,7 +361,10 @@ class LinearGrid(Grid):
             self._M = t.size
         else:
             t = np.linspace(0, 2 * np.pi, self.M, endpoint=endpoint)
-        dt = 2 * np.pi / self.M
+        if self.M > 0:
+            dt = 2 * np.pi / self.M
+        else:
+            dt = 2 * np.pi
 
         # zeta/phi
         if zeta is not None:
@@ -346,7 +372,10 @@ class LinearGrid(Grid):
             self._N = z.size
         else:
             z = np.linspace(0, 2 * np.pi / self.NFP, self.N, endpoint=endpoint)
-        dz = 2 * np.pi / self.NFP / self.N
+        if self.N > 0:
+            dz = 2 * np.pi / self.NFP / self.N
+        else:
+            dz = 2 * np.pi / self.NFP
 
         r, t, z = np.meshgrid(r, t, z, indexing="ij")
         r = r.flatten()
@@ -358,12 +387,12 @@ class LinearGrid(Grid):
         dz = dz * np.ones_like(z)
 
         nodes = np.stack([r, t, z]).T
-        weights = dr * dt * dz
+        spacing = np.stack([dr, dt, dz]).T
 
-        return nodes, weights
+        return nodes, spacing
 
-    def change_resolution(self, L, M, N):
-        """Change the resolution of the grid
+    def change_resolution(self, L, M, N, NFP=None):
+        """Change the resolution of the grid.
 
         Parameters
         ----------
@@ -373,25 +402,32 @@ class LinearGrid(Grid):
             new poloidal grid resolution (M poloidal nodes)
         N : int
             new toroidal grid resolution (N toroidal nodes)
+        NFP : int
+            Number of field periods.
 
         """
+        self._NFP = NFP if NFP is not None else self.NFP
         if L != self.L or M != self.M or N != self.N:
             self._L = L
             self._M = M
             self._N = N
-            self._nodes, self._weights = self._create_nodes(
-                L=L, M=M, N=N, NFP=self.NFP, axis=self.axis, endpoint=self.endpoint
+            self._nodes, self._spacing = self._create_nodes(
+                L=L,
+                M=M,
+                N=N,
+                NFP=self.NFP,
+                axis=len(self.axis) > 0,
+                endpoint=self.endpoint,
             )
             self._enforce_symmetry()
             self._sort_nodes()
             self._find_axis()
+            self._scale_weights()
 
     @property
     def endpoint(self):
-        """bool: whether the grid is made of open or closed intervals"""
-        if not hasattr(self, "_endpoint"):
-            self._endpoint = False
-        return self._endpoint
+        """bool: Whether the grid is made of open or closed intervals."""
+        return self.__dict__.setdefault("_endpoint", False)
 
 
 class QuadratureGrid(Grid):
@@ -422,17 +458,18 @@ class QuadratureGrid(Grid):
         self._sym = False
         self._node_pattern = "quad"
 
-        self._nodes, self._weights = self._create_nodes(
+        self._nodes, self._spacing = self._create_nodes(
             L=self.L, M=self.M, N=self.N, NFP=self.NFP
         )
 
         self._enforce_symmetry()  # symmetry is never enforced for Quadrature Grid
         self._sort_nodes()
         self._find_axis()
-        # quad grid should already be exact, so we don't scale weights
+        self._count_nodes()
+        self._weights = self.spacing.prod(axis=1)  # Quad weights don't need scaling
 
     def _create_nodes(self, L=1, M=1, N=1, NFP=1):
-        """
+        """Create grid nodes and weights.
 
         Parameters
         ----------
@@ -449,8 +486,8 @@ class QuadratureGrid(Grid):
         -------
         nodes : ndarray of float, size(num_nodes,3)
             node coordinates, in (rho,theta,zeta)
-        weights : ndarray of float, size(num_nodes,)
-            weight for each node, based on local volume around the node
+        spacing : ndarray of float, size(num_nodes,3)
+            node spacing, based on local volume around the node
 
         """
         self._L = L
@@ -485,12 +522,12 @@ class QuadratureGrid(Grid):
         wr /= r  # remove r weight function associated with the shifted Jacobi weights
 
         nodes = np.stack([r, t, z]).T
-        weights = wr * wt * wz
+        spacing = np.stack([wr, wt, wz]).T
 
-        return nodes, weights
+        return nodes, spacing
 
-    def change_resolution(self, L, M, N):
-        """Change the resolution of the grid
+    def change_resolution(self, L, M, N, NFP=None):
+        """Change the resolution of the grid.
 
         Parameters
         ----------
@@ -500,20 +537,24 @@ class QuadratureGrid(Grid):
             new poloidal grid resolution (M poloidal nodes)
         N : int
             new toroidal grid resolution (N toroidal nodes)
+        NFP : int
+            Number of field periods.
 
         """
+        self._NFP = NFP if NFP is not None else self.NFP
         if L != self.L or M != self.M or N != self.N:
             self._L = L
             self._M = M
             self._N = N
-            self._nodes, self._weights = self._create_nodes(L=L, M=M, N=N, NFP=self.NFP)
+            self._nodes, self._spacing = self._create_nodes(L=L, M=M, N=N, NFP=self.NFP)
             self._enforce_symmetry()
             self._sort_nodes()
             self._find_axis()
+            self._weights = self.spacing.prod(axis=1)  # instead of _scale_weights
 
 
 class ConcentricGrid(Grid):
-    """Grid in which the nodes are arranged in concentric circles
+    """Grid in which the nodes are arranged in concentric circles.
 
     Nodes are arranged concentrically within each toroidal cross-section, with more
     nodes per flux surface at larger radius. Typically used as the solution grid,
@@ -533,21 +574,34 @@ class ConcentricGrid(Grid):
         True for stellarator symmetry, False otherwise (Default = False)
     axis : bool
         True to include the magnetic axis, False otherwise (Default = False)
+    rotation : {``'cos'``, ``'sin'``, False}
+        * ``'cos'`` for cos(m*t-n*z) symmetry, gives nodes at theta=0
+        * ``'sin'`` for sin(m*t-n*z) symmetry, gives nodes at theta=pi/2
+        * ``None`` for no symmetry (Default), rotates halfway between other options
     node_pattern : {``'cheb1'``, ``'cheb2'``, ``'jacobi'``, ``None``}
         pattern for radial coordinates
 
             * ``'cheb1'``: Chebyshev-Gauss-Lobatto nodes scaled to r=[0,1]
             * ``'cheb2'``: Chebyshev-Gauss-Lobatto nodes scaled to r=[-1,1]
-            * ``'jacobi'``: Radial nodes are roots of Shifted Jacobi polynomial of degree
-              M+1 r=(0,1), and angular nodes are equispaced 2(M+1) per surface
+            * ``'jacobi'``: Radial nodes are roots of Shifted Jacobi polynomial of
+              degree M+1 r=(0,1), and angular nodes are equispaced 2(M+1) per surface
             * ``'ocs'``: optimal concentric sampling to minimize the condition number
               of the resulting transform matrix, for doing inverse transform.
             * ``None`` : linear spacing in r=[0,1]
 
-
     """
 
-    def __init__(self, L, M, N, NFP=1, sym=False, axis=False, node_pattern="jacobi"):
+    def __init__(
+        self,
+        L,
+        M,
+        N,
+        NFP=1,
+        sym=False,
+        axis=False,
+        rotation=None,
+        node_pattern="jacobi",
+    ):
 
         self._L = L
         self._M = M
@@ -555,24 +609,29 @@ class ConcentricGrid(Grid):
         self._NFP = NFP
         self._sym = sym
         self._axis = axis
+        self._rotation = rotation
         self._node_pattern = node_pattern
 
-        self._nodes, self._weights = self._create_nodes(
+        self._nodes, self._spacing = self._create_nodes(
             L=self.L,
             M=self.M,
             N=self.N,
             NFP=self.NFP,
             axis=self.axis,
+            rotation=self._rotation,
             node_pattern=self.node_pattern,
         )
 
         self._enforce_symmetry()
         self._sort_nodes()
         self._find_axis()
+        self._count_nodes()
         self._scale_weights()
 
-    def _create_nodes(self, L, M, N, NFP=1, axis=False, node_pattern="jacobi"):
-        """
+    def _create_nodes(
+        self, L, M, N, NFP=1, axis=False, rotation=None, node_pattern="jacobi"
+    ):
+        """Create grid nodes and weights.
 
         Parameters
         ----------
@@ -586,28 +645,32 @@ class ConcentricGrid(Grid):
             number of field periods (Default = 1)
         axis : bool
             True to include the magnetic axis, False otherwise (Default = False)
-        node_pattern : {``'cheb1'``, ``'cheb2'``, ``'jacobi'``, ``None``}
+        rotation : {``'cos'``, ``'sin'``, False}
+            * ``'cos'`` for cos(m*t-n*z) symmetry, gives nodes at theta=0
+            * ``'sin'`` for sin(m*t-n*z) symmetry, gives nodes at theta=pi/2
+            * ``None`` for no symmetry (Default), rotates halfway between other options
+        node_pattern : {``'linear'``, ``'cheb1'``, ``'cheb2'``, ``'jacobi'``, ``None``}
             pattern for radial coordinates
-
+                * ``linear`` : linear spacing in r=[0,1]
                 * ``'cheb1'``: Chebyshev-Gauss-Lobatto nodes scaled to r=[0,1]
                 * ``'cheb2'``: Chebyshev-Gauss-Lobatto nodes scaled to r=[-1,1]
-                * ``'jacobi'``: Radial nodes are roots of Shifted Jacobi polynomial of degree
-                M+1 r=(0,1), and angular nodes are equispaced 2(M+1) per surface
-                * ``'ocs'``: optimal concentric sampling to minimize the condition number
-                  of the resulting transform matrix, for doing inverse transform.
-                * ``None`` : linear spacing in r=[0,1]
+                * ``'jacobi'``: Radial nodes are roots of Shifted Jacobi polynomial of
+                  degree M+1 r=(0,1), and angular nodes are equispaced 2(M+1) per
+                  surface.
+                * ``'ocs'``: optimal concentric sampling to minimize the condition
+                  number of the resulting transform matrix, for doing inverse transform.
 
         Returns
         -------
         nodes : ndarray of float, size(num_nodes, 3)
             node coordinates, in (rho,theta,zeta)
-        weights : ndarray of float, size(num_nodes,)
-            weight for each node, either exact quadrature or volume based
+        spacing : ndarray of float, size(num_nodes,3)
+            node spacing, based on local volume around the node
 
         """
 
         def ocs(L):
-            # from Ramos-Lopez, et al “Optimal Sampling Patterns for Zernike Polynomials.”
+            # Ramos-Lopez, et al “Optimal Sampling Patterns for Zernike Polynomials.”
             # Applied Mathematics and Computation 274 (February 2016): 247–57.
             # https://doi.org/10.1016/j.amc.2015.11.006.
             j = np.arange(1, L // 2 + 2)
@@ -616,12 +679,15 @@ class ConcentricGrid(Grid):
             return np.sort(rj)
 
         pattern = {
+            "linear": np.linspace(0, 1, num=L // 2 + 1),
             "cheb1": (np.cos(np.arange(L // 2, -1, -1) * np.pi / (L // 2)) + 1) / 2,
             "cheb2": -np.cos(np.arange(L // 2, L + 1, 1) * np.pi / L),
             "jacobi": special.js_roots(L // 2 + 1, 2, 2)[0],
             "ocs": ocs(L),
         }
-        rho = pattern.get(node_pattern, np.linspace(0, 1, num=L // 2 + 1))
+        rho = pattern.get(node_pattern)
+        if rho is None:
+            raise ValueError("node_pattern '{}' is not supported".format(node_pattern))
         rho = np.sort(rho, axis=None)
         if axis:
             rho[0] = 0
@@ -629,26 +695,40 @@ class ConcentricGrid(Grid):
             rho[0] = rho[1] / 10
 
         drho = np.zeros_like(rho)
-        for i in range(rho.size):
-            if i == 0:
-                drho[i] = (rho[0] + rho[1]) / 2
-            elif i == rho.size - 1:
-                drho[i] = 1 - (rho[-2] + rho[-1]) / 2
-            else:
-                drho[i] = (rho[i + 1] - rho[i - 1]) / 2
-
+        if rho.size > 1:
+            for i in range(rho.size):
+                if i == 0:
+                    drho[i] = (rho[0] + rho[1]) / 2
+                elif i == rho.size - 1:
+                    drho[i] = 1 - (rho[-2] + rho[-1]) / 2
+                else:
+                    drho[i] = (rho[i + 1] - rho[i - 1]) / 2
+        else:
+            drho = np.array([1.0])
         r = []
         t = []
         dr = []
         dt = []
 
         for iring in range(L // 2 + 1, 0, -1):
-            dtheta = (
-                2 * np.pi / (2 * M + np.ceil((M / L) * (5 - 4 * iring)).astype(int))
-            )
-            theta = np.arange(0, 2 * np.pi, dtheta)
-            if self.sym:
-                theta = (theta + dtheta / 3) % (2 * np.pi)
+            ntheta = 2 * M + np.ceil((M / L) * (5 - 4 * iring)).astype(int)
+            dtheta = 2 * np.pi / ntheta
+            theta = np.linspace(0, 2 * np.pi, ntheta, endpoint=False)
+            if rotation in {None, False}:
+                if self.sym:
+                    # this is emperically chosen, could be something different, just
+                    # need to avoid symmetry at theta=0, pi
+                    offset = dtheta / 3
+                else:
+                    offset = 0
+            elif rotation in ["cos", "cosine"]:  # cos(m*t-n*z) symmetry
+                offset = 0
+            elif rotation in ["sin", "sine"]:  # sin(m*t-n*z) symmetry
+                offset = dtheta / 4
+            else:
+                raise ValueError(f"Unknown rotation type {rotation}")
+
+            theta = (theta + offset) % (2 * np.pi)
             for tk in theta:
                 r.append(rho[-iring])
                 t.append(tk)
@@ -662,7 +742,7 @@ class ConcentricGrid(Grid):
         dimzern = r.size
 
         dz = 2 * np.pi / (NFP * (2 * N + 1))
-        z = np.arange(0, 2 * np.pi / NFP, dz)
+        z = np.linspace(0, 2 * np.pi / NFP, 2 * N + 1, endpoint=False)
 
         r = np.tile(r, 2 * N + 1)
         t = np.tile(t, 2 * N + 1)
@@ -671,12 +751,12 @@ class ConcentricGrid(Grid):
         dt = np.tile(dt, 2 * N + 1)
         dz = np.ones_like(z) * dz
         nodes = np.stack([r, t, z]).T
-        weights = dr * dt * dz
+        spacing = np.stack([dr, dt, dz]).T
 
-        return nodes, weights
+        return nodes, spacing
 
-    def change_resolution(self, L, M, N):
-        """Change the resolution of the grid
+    def change_resolution(self, L, M, N, NFP=None):
+        """Change the resolution of the grid.
 
         Parameters
         ----------
@@ -686,18 +766,27 @@ class ConcentricGrid(Grid):
             new poloidal grid resolution
         N : int
             new toroidal grid resolution
+        NFP : int
+            Number of field periods.
 
         """
+        self._NFP = NFP if NFP is not None else self.NFP
         if L != self.L or M != self.M or N != self.N:
             self._L = L
             self._M = M
             self._N = N
-            self._nodes, self._weights = self._create_nodes(
-                L=L, M=M, N=N, NFP=self.NFP, node_pattern=self.node_pattern
+            self._nodes, self._spacing = self._create_nodes(
+                L=L,
+                M=M,
+                N=N,
+                NFP=self.NFP,
+                axis=len(self.axis) > 0,
+                node_pattern=self.node_pattern,
             )
             self._enforce_symmetry()
             self._sort_nodes()
             self._find_axis()
+            self._scale_weights()
 
 
 # these functions are currently unused ---------------------------------------
@@ -713,7 +802,8 @@ def dec_to_cf(x, dmax=6):  # pragma: no cover
     x : float
         floating point form of number
     dmax : int
-        maximum iterations (ie, number of coefficients of continued fraction). (Default value = 6)
+        maximum iterations (ie, number of coefficients of continued fraction).
+        (Default value = 6)
 
     Returns
     -------
@@ -755,7 +845,7 @@ def cf_to_dec(cf):  # pragma: no cover
 
 
 def most_rational(a, b):  # pragma: no cover
-    """Compute the most rational number in the range [a,b]
+    """Compute the most rational number in the range [a,b].
 
     Parameters
     ----------
