@@ -56,7 +56,8 @@ def load(load_from, file_format=None):
             raise ValueError(
                 "Could not load from {}, no __class__ attribute found".format(load_from)
             )
-
+    else:
+        raise ValueError("Unknown file format: {}".format(file_format))
     # to set other secondary stuff that wasnt saved possibly:
     if hasattr(obj, "_set_up"):
         obj._set_up()
@@ -197,6 +198,23 @@ class IOAble(ABC):
             dict2 = other.__dict__
         return equals(dict1, dict2)
 
+    def __copy__(self):
+        cls = self.__class__
+        result = cls.__new__(cls)
+        result.__dict__.update(self.__dict__)
+        return result
+
+    def __deepcopy__(self, memo):
+        cls = self.__class__
+        result = cls.__new__(cls)
+        memo[id(self)] = result
+        for k, v in self.__dict__.items():
+            try:
+                setattr(result, k, copy.deepcopy(v, memo))
+            except TypeError:
+                setattr(result, k, copy.copy(v))
+        return result
+
     def copy(self, deepcopy=True):
         """Return a (deep)copy of this object."""
         if deepcopy:
