@@ -137,7 +137,8 @@ class hdf5Reader(hdf5IO, Reader):
                             obj,
                             attr,
                             cls.load(
-                                load_from=loc[attr], file_format=self._file_format_,
+                                load_from=loc[attr],
+                                file_format=self._file_format_,
                             ),
                         )
                     else:
@@ -184,7 +185,8 @@ class hdf5Reader(hdf5IO, Reader):
                     cls = pydoc.locate(cls_name)
                     if cls is not None:
                         thedict[key] = cls.load(
-                            load_from=loc[key], file_format=self._file_format_,
+                            load_from=loc[key],
+                            file_format=self._file_format_,
                         )
                     else:
                         warnings.warn(
@@ -236,7 +238,8 @@ class hdf5Reader(hdf5IO, Reader):
                     if cls is not None:
                         thelist.append(
                             cls.load(
-                                load_from=loc[str(i)], file_format=self._file_format_,
+                                load_from=loc[str(i)],
+                                file_format=self._file_format_,
                             ),
                         )
                     else:
@@ -283,6 +286,9 @@ class hdf5Writer(hdf5IO, Writer):
 
         # save name of object class
         loc.create_dataset("__class__", data=fullname(obj))
+        from desc import __version__
+
+        loc.create_dataset("__version__", data=__version__)
         for attr in obj._io_attrs_:
             try:
                 data = getattr(obj, attr)
