@@ -3,6 +3,7 @@ import pathlib
 import warnings
 import os
 import re
+import io
 import numpy as np
 from datetime import datetime
 from termcolor import colored
@@ -168,11 +169,12 @@ class InputReader:
             "nfev": np.atleast_1d(None),
             "objective": "force",
             "optimizer": "lsq-exact",
-            "spectral_indexing": "fringe",
+            "spectral_indexing": "ansi",
             "node_pattern": "jacobi",
             "bdry_mode": "lcfs",
-            "profiles": np.atleast_2d((0, 0.0, 0.0)),
-            "boundary": np.atleast_2d((0, 0, 0, 0.0, 0.0)),
+            "pressure": np.atleast_2d((0, 0.0)),
+            "iota": np.atleast_2d((0, 0.0)),
+            "surface": np.atleast_2d((0, 0, 0, 0.0, 0.0)),
             "axis": np.atleast_2d((0, 0.0, 0.0)),
         }
 
@@ -185,7 +187,13 @@ class InputReader:
         else:
             inputs["verbose"] = 1
 
-        file = open(fname, "r")
+        # open files, unless they are already open files
+        if not isinstance(fname, io.IOBase):
+            file = open(fname, "r")
+        else:
+            file = fname
+        file.seek(0)
+
         num_form = r"[-+]?\ *\d*\.?\d*(?:[Ee]\ *[-+]?\ *\d+)?"
 
         for line in file:
@@ -195,7 +203,7 @@ class InputReader:
             if isVMEC:
                 print("Converting VMEC input to DESC input")
                 path = self.input_path + "_desc"
-                self.vmec_to_desc_input(self.input_path, path)
+                InputReader.vmec_to_desc_input(self.input_path, path)
                 print("Generated DESC input file {}:".format(path))
                 return self.parse_inputs(path)
 
@@ -382,14 +390,14 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ][0]
-                prof_idx = np.where(inputs["profiles"][:, 0] == l)[0]
+                prof_idx = np.where(inputs["pressure"][:, 0] == l)[0]
                 if prof_idx.size == 0:
-                    prof_idx = np.atleast_1d(inputs["profiles"].shape[0])
-                    inputs["profiles"] = np.pad(
-                        inputs["profiles"], ((0, 1), (0, 0)), mode="constant"
+                    prof_idx = np.atleast_1d(inputs["pressure"].shape[0])
+                    inputs["pressure"] = np.pad(
+                        inputs["pressure"], ((0, 1), (0, 0)), mode="constant"
                     )
-                    inputs["profiles"][prof_idx[0], 0] = l
-                inputs["profiles"][prof_idx[0], 1] = p_l
+                    inputs["pressure"][prof_idx[0], 0] = l
+                inputs["pressure"][prof_idx[0], 1] = p_l
                 flag = True
             match = re.search(r"\si\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
@@ -398,14 +406,14 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ][0]
-                prof_idx = np.where(inputs["profiles"][:, 0] == l)[0]
+                prof_idx = np.where(inputs["iota"][:, 0] == l)[0]
                 if prof_idx.size == 0:
-                    prof_idx = np.atleast_1d(inputs["profiles"].shape[0])
-                    inputs["profiles"] = np.pad(
-                        inputs["profiles"], ((0, 1), (0, 0)), mode="constant"
+                    prof_idx = np.atleast_1d(inputs["iota"].shape[0])
+                    inputs["iota"] = np.pad(
+                        inputs["iota"], ((0, 1), (0, 0)), mode="constant"
                     )
-                    inputs["profiles"][prof_idx[0], 0] = l
-                inputs["profiles"][prof_idx[0], 2] = i_l
+                    inputs["iota"][prof_idx[0], 0] = l
+                inputs["iota"][prof_idx[0], 1] = i_l
                 flag = True
 
             # boundary surface coefficients
@@ -417,17 +425,17 @@ class InputReader:
                     if re.search(r"\d", x)
                 ][1]
                 bdry_idx = np.where(
-                    (inputs["boundary"][:, :3] == [l, m, n]).all(axis=1)
+                    (inputs["surface"][:, :3] == [l, m, n]).all(axis=1)
                 )[0]
                 if bdry_idx.size == 0:
-                    bdry_idx = np.atleast_1d(inputs["boundary"].shape[0])
-                    inputs["boundary"] = np.pad(
-                        inputs["boundary"], ((0, 1), (0, 0)), mode="constant"
+                    bdry_idx = np.atleast_1d(inputs["surface"].shape[0])
+                    inputs["surface"] = np.pad(
+                        inputs["surface"], ((0, 1), (0, 0)), mode="constant"
                     )
-                    inputs["boundary"][bdry_idx[0], 0] = l
-                    inputs["boundary"][bdry_idx[0], 1] = m
-                    inputs["boundary"][bdry_idx[0], 2] = n
-                inputs["boundary"][bdry_idx[0], 3] = R1
+                    inputs["surface"][bdry_idx[0], 0] = l
+                    inputs["surface"][bdry_idx[0], 1] = m
+                    inputs["surface"][bdry_idx[0], 2] = n
+                inputs["surface"][bdry_idx[0], 3] = R1
                 flag = True
             match = re.search(r"Z1\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
@@ -437,17 +445,17 @@ class InputReader:
                     if re.search(r"\d", x)
                 ][1]
                 bdry_idx = np.where(
-                    (inputs["boundary"][:, :3] == [l, m, n]).all(axis=1)
+                    (inputs["surface"][:, :3] == [l, m, n]).all(axis=1)
                 )[0]
                 if bdry_idx.size == 0:
-                    bdry_idx = np.atleast_1d(inputs["boundary"].shape[0])
-                    inputs["boundary"] = np.pad(
-                        inputs["boundary"], ((0, 1), (0, 0)), mode="constant"
+                    bdry_idx = np.atleast_1d(inputs["surface"].shape[0])
+                    inputs["surface"] = np.pad(
+                        inputs["surface"], ((0, 1), (0, 0)), mode="constant"
                     )
-                    inputs["boundary"][bdry_idx[0], 0] = l
-                    inputs["boundary"][bdry_idx[0], 1] = m
-                    inputs["boundary"][bdry_idx[0], 2] = n
-                inputs["boundary"][bdry_idx[0], 4] = Z1
+                    inputs["surface"][bdry_idx[0], 0] = l
+                    inputs["surface"][bdry_idx[0], 1] = m
+                    inputs["surface"][bdry_idx[0], 2] = n
+                inputs["surface"][bdry_idx[0], 4] = Z1
                 flag = True
 
             # magnetic axis coefficients
@@ -495,7 +503,7 @@ class InputReader:
         # error handling
         if np.any(inputs["M"] == 0):
             raise IOError(colored("M_pol is not assigned", "red"))
-        if np.sum(inputs["boundary"]) == 0:
+        if np.sum(inputs["surface"]) == 0:
             raise IOError(colored("Fixed-boundary surface is not assigned", "red"))
         arrs = [
             "L",
@@ -523,12 +531,12 @@ class InputReader:
 
         # unsupplied values
         if np.sum(inputs["M_grid"]) == 0:
-            inputs["M_grid"] = np.rint(1.5 * inputs["M"]).astype(int)
+            inputs["M_grid"] = (2 * inputs["M"]).astype(int)
         if np.sum(inputs["N_grid"]) == 0:
-            inputs["N_grid"] = np.rint(1.5 * inputs["N"]).astype(int)
+            inputs["N_grid"] = (2 * inputs["N"]).astype(int)
         if np.sum(inputs["axis"]) == 0:
-            axis_idx = np.where(inputs["boundary"][:, 1] == 0)[0]
-            inputs["axis"] = inputs["boundary"][axis_idx, 2:]
+            axis_idx = np.where(inputs["surface"][:, 1] == 0)[0]
+            inputs["axis"] = inputs["surface"][axis_idx, 2:]
         if None in inputs["L"]:
             default_L = {
                 "ansi": inputs["M"],
@@ -554,33 +562,43 @@ class InputReader:
                 else:
                     inputs_ii[key] = inputs[key]
             # apply pressure ratio
-            inputs_ii["profiles"][:, 1] *= inputs_ii["pres_ratio"]
+            inputs_ii["pressure"][:, 1] *= inputs_ii["pres_ratio"]
             # apply boundary ratio
             bdry_factor = np.where(
-                inputs_ii["boundary"][:, 2] != 0, inputs_ii["bdry_ratio"], 1
+                inputs_ii["surface"][:, 2] != 0, inputs_ii["bdry_ratio"], 1
             )
-            inputs_ii["boundary"][:, 3] *= bdry_factor
-            inputs_ii["boundary"][:, 4] *= bdry_factor
+            inputs_ii["surface"][:, 3] *= bdry_factor
+            inputs_ii["surface"][:, 4] *= bdry_factor
             inputs_list.append(inputs_ii)
 
         return inputs_list
 
-    def write_desc_input(self, filename, inputs=None):
+    @staticmethod
+    def write_desc_input(filename, inputs, header=""):
         """Generate a DESC input file from a dictionary of parameters.
 
         Parameters
         ----------
         filename : str or path-like
             name of the file to create
-        inputs : dict
+        inputs : dict or list of dict
             dictionary of input parameters
+        header : str
+            text to print at the top of the file
 
         """
-        # default to use self.inputs
-        if inputs is None:
-            inputs = self.inputs
 
-        f = open(filename, "w+")
+        # open the file, unless its already open
+        if not isinstance(filename, io.IOBase):
+            f = open(filename, "w+")
+        else:
+            f = filename
+        f.seek(0)
+
+        if not isinstance(inputs, (list, tuple)):
+            inputs = [inputs]
+
+        f.write(header + "\n")
 
         f.write("# global parameters \n")
         f.write("sym = {} \n".format(inputs[0]["sym"]))
@@ -628,11 +646,24 @@ class InputReader:
         f.write("node_pattern = {} \n".format(inputs[0]["node_pattern"]))
 
         f.write("\n# pressure and rotational transform profiles \n")
-        for (l, p, i) in inputs[0]["profiles"]:
+        ls = np.unique(
+            np.concatenate([inputs[0]["pressure"][:, 0], inputs[0]["iota"][:, 0]])
+        )
+        for l in ls:
+            idxp = np.where(l == inputs[0]["pressure"][:, 0])[0]
+            if len(idxp):
+                p = inputs[0]["pressure"][idxp[0], 1]
+            else:
+                p = 0.0
+            idxi = np.where(l == inputs[0]["iota"][:, 0])[0]
+            if len(idxi):
+                i = inputs[0]["iota"][idxi[0], 1]
+            else:
+                i = 0.0
             f.write("l: {:3d}\tp = {:16.8E}\ti = {:16.8E}\n".format(int(l), p, i))
 
         f.write("\n# fixed-boundary surface shape \n")
-        for (l, m, n, R1, Z1) in inputs[0]["boundary"]:
+        for (l, m, n, R1, Z1) in inputs[0]["surface"]:
             f.write(
                 "l: {:3d}\tm: {:3d}\tn: {:3d}\tR1 = {:16.8E}\tZ1 = {:16.8E}\n".format(
                     int(l), int(m), int(n), R1, Z1
@@ -645,7 +676,8 @@ class InputReader:
 
         f.close()
 
-    def vmec_to_desc_input(self, vmec_fname, desc_fname):
+    @staticmethod
+    def vmec_to_desc_input(vmec_fname, desc_fname):
         """Convert a VMEC input file to an equivalent DESC input file.
 
         Parameters
@@ -656,28 +688,72 @@ class InputReader:
             filename of DESC input file. If it already exists it is overwritten.
 
         """
-        # file objects
-        vmec_file = open(vmec_fname, "r")
-        desc_file = open(desc_fname, "w")
-
-        desc_file.seek(0)
         now = datetime.now()
         date = now.strftime("%m/%d/%Y")
         time = now.strftime("%H:%M:%S")
-        desc_file.write(
+        header = (
             "# This DESC input file was auto generated from the VMEC input file\n"
-            + "# {}\n# on {} at {}.\n\n".format(vmec_fname, date, time)
+            + "# {}\n# on {} at {}.\n".format(vmec_fname, date, time)
+            + "# For details on the various options see https://desc-docs.readthedocs.io/en/stable/input.html\n"
         )
+        inputs = InputReader.parse_vmec_inputs(vmec_fname)
+        InputReader.write_desc_input(desc_fname, inputs, header)
 
+    @staticmethod
+    def parse_vmec_inputs(vmec_fname):
+        """Parse a VMEC input file into a dictionary of DESC inputs
+
+        Parameters
+        ----------
+        vmec_fname : str or PathLike
+            path to VMEC input file
+
+        Returns
+        -------
+        inputs : dict
+            dictionary of inputs formatted for DESC
+
+        """
+
+        if not isinstance(vmec_fname, io.IOBase):
+            vmec_file = open(vmec_fname, "r")
+        else:
+            vmec_file = vmec_fname
+
+        vmec_file.seek(0)
         num_form = r"[-+]?\ *\d*\.?\d*(?:[Ee]\ *[-+]?\ *\d+)?"
         Ntor = 99
 
-        pres_scale = 1.0
-        p_l = np.array([0.0])
-        i_l = np.array([0.0])
-        axis = np.array([[0, 0, 0.0]])
-        bdry = np.array([[0, 0, 0.0, 0.0]])
+        # default values
+        inputs = {
+            "sym": False,
+            "NFP": 1,
+            "Psi": 1.0,
+            "L": None,
+            "M": 0,
+            "N": 0,
+            "L_grid": None,
+            "M_grid": 0,
+            "N_grid": 0,
+            "pres_ratio": 1.0,
+            "bdry_ratio": 1.0,
+            "pert_order": 1,
+            "ftol": 1e-2,
+            "xtol": 1e-6,
+            "gtol": 1e-6,
+            "nfev": None,
+            "objective": "force",
+            "optimizer": "lsq-exact",
+            "spectral_indexing": "ansi",
+            "node_pattern": "jacobi",
+            "bdry_mode": "lcfs",
+            "pressure": np.atleast_2d((0, 0.0)),
+            "iota": np.atleast_2d((0, 0.0)),
+            "surface": np.atleast_2d((0, 0, 0.0, 0.0)),
+            "axis": np.atleast_2d((0, 0.0, 0.0)),
+        }
 
+        pres_scale = 1.0
         for line in vmec_file:
             comment = line.find("!")
             command = (line.strip() + " ")[0:comment]
@@ -687,12 +763,12 @@ class InputReader:
                 warnings.warn(
                     colored("Using poloidal flux instead of toroidal flux!", "yellow")
                 )
-            match = re.search("LASYM\s*=\s*[TF]", command, re.IGNORECASE)
+            match = re.search(r"LASYM\s*=\s*[TF]", command, re.IGNORECASE)
             if match:
                 if re.search(r"T", match.group(0), re.IGNORECASE):
-                    desc_file.write("sym = 0\n")
+                    inputs["sym"] = 0
                 else:
-                    desc_file.write("sym = 1\n")
+                    inputs["sym"] = 1
             match = re.search(r"NFP\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
                 numbers = [
@@ -700,7 +776,7 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ]
-                desc_file.write("NFP = {:3d}\n".format(numbers[0]))
+                inputs["NFP"] = numbers[0]
             match = re.search(r"PHIEDGE\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
                 numbers = [
@@ -708,7 +784,7 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ]
-                desc_file.write("Psi = {:16.8E}\n".format(numbers[0]))
+                inputs["Psi"] = numbers[0]
             match = re.search(r"MPOL\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
                 numbers = [
@@ -716,7 +792,10 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ]
-                desc_file.write("M_pol = {:3d}\n".format(numbers[0]))
+                inputs["M"] = numbers[0]
+                inputs["L"] = numbers[0]
+                inputs["L_grid"] = 2 * numbers[0]
+                inputs["M_grid"] = 2 * numbers[0]
             match = re.search(r"NTOR\s*=\s*" + num_form, command, re.IGNORECASE)
             if match:
                 numbers = [
@@ -724,7 +803,8 @@ class InputReader:
                     for x in re.findall(num_form, match.group(0))
                     if re.search(r"\d", x)
                 ]
-                desc_file.write("N_tor = {:3d}\n".format(numbers[0]))
+                inputs["N"] = numbers[0]
+                inputs["N_grid"] = 2 * numbers[0]
                 Ntor = numbers[0]
 
             # pressure profile
@@ -776,9 +856,14 @@ class InputReader:
                 ]
                 for k in range(len(numbers)):
                     l = 2 * k
-                    if p_l.size < l + 1:
-                        p_l = np.pad(p_l, (0, l + 1 - p_l.size), mode="constant")
-                    p_l[l] = numbers[k]
+                    if len(inputs["pressure"]) < l + 1:
+                        inputs["pressure"] = np.pad(
+                            inputs["pressure"],
+                            ((0, l + 1 - len(inputs["pressure"])), (0, 0)),
+                            mode="constant",
+                        )
+                    inputs["pressure"][l, 1] = numbers[k]
+                    inputs["pressure"][l, 0] = l
 
             # rotational transform
             match = re.search(
@@ -804,9 +889,14 @@ class InputReader:
                 ]
                 for k in range(len(numbers)):
                     l = 2 * k
-                    if i_l.size < l + 1:
-                        i_l = np.pad(i_l, (0, l + 1 - i_l.size), mode="constant")
-                    i_l[l] = numbers[k]
+                    if len(inputs["iota"]) < l + 1:
+                        inputs["iota"] = np.pad(
+                            inputs["iota"],
+                            ((0, l + 1 - len(inputs["iota"])), (0, 0)),
+                            mode="constant",
+                        )
+                    inputs["iota"][l, 1] = numbers[k]
+                    inputs["iota"][l, 0] = l
 
             # magnetic axis
             match = re.search(
@@ -823,12 +913,14 @@ class InputReader:
                         l = -k + Ntor + 1
                     else:
                         l = k
-                    idx = np.where(axis[:, 0] == l)[0]
+                    idx = np.where(inputs["axis"][:, 0] == l)[0]
                     if np.size(idx):
-                        axis[idx[0], 1] += numbers[k]
+                        inputs["axis"][idx[0], 1] += numbers[k]
                     else:
-                        axis = np.pad(axis, ((0, 1), (0, 0)), mode="constant")
-                        axis[-1, :] = np.array([l, numbers[k], 0.0])
+                        inputs["axis"] = np.pad(
+                            inputs["axis"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["axis"][-1, :] = np.array([l, numbers[k], 0.0])
             match = re.search(
                 r"ZAXIS\s*=(\s*" + num_form + ")*", command, re.IGNORECASE
             )
@@ -843,11 +935,11 @@ class InputReader:
                         l = k - Ntor - 1
                     else:
                         l = -k
-                    idx = np.where(axis[:, 0] == l)[0]
+                    idx = np.where(inputs["axis"][:, 0] == l)[0]
                     if np.size(idx) > 0:
-                        axis[idx[0], 2] += numbers[k]
+                        inputs["axis"][idx[0], 2] += numbers[k]
                     else:
-                        axis = np.pad(axis, ((0, 1), (0, 0)), mode="constant")
+                        axis = np.pad(inputs["axis"], ((0, 1), (0, 0)), mode="constant")
                         axis[-1, :] = np.array([l, 0.0, numbers[k]])
 
             # boundary shape
@@ -855,9 +947,9 @@ class InputReader:
             match = re.search(
                 r"RBS\(\s*"
                 + num_form
-                + "\s*,\s*"
+                + r"\s*,\s*"
                 + num_form
-                + "\s*\)\s*=\s*"
+                + r"\s*\)\s*=\s*"
                 + num_form,
                 command,
                 re.IGNORECASE,
@@ -878,30 +970,34 @@ class InputReader:
                     m = abs(m)
                 RBS = numbers[2]
                 if m != 0:  # RBS*sin(m*t)*cos(n*p)
-                    m_idx = np.where(bdry[:, 0] == -m)[0]
-                    n_idx = np.where(bdry[:, 1] == n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == -m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 2] += m_sgn * RBS
+                        inputs["surface"][m_idx[idx[0]], 2] += m_sgn * RBS
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([-m, n, m_sgn * RBS, 0.0])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array([-m, n, m_sgn * RBS, 0.0])
                 if n != 0:  # -RBS*cos(m*t)*sin(n*p)
-                    m_idx = np.where(bdry[:, 0] == m)[0]
-                    n_idx = np.where(bdry[:, 1] == -n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == -n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 2] += -n_sgn * RBS
+                        inputs["surface"][m_idx[idx[0]], 2] += -n_sgn * RBS
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([m, -n, -n_sgn * RBS, 0.0])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array([m, -n, -n_sgn * RBS, 0.0])
             # RBC*cos(m*t-n*p) = RBC*cos(m*t)*cos(n*p) + RBC*sin(m*t)*sin(n*p)
             match = re.search(
                 r"RBC\(\s*"
                 + num_form
-                + "\s*,\s*"
+                + r"\s*,\s*"
                 + num_form
-                + "\s*\)\s*=\s*"
+                + r"\s*\)\s*=\s*"
                 + num_form,
                 command,
                 re.IGNORECASE,
@@ -922,30 +1018,36 @@ class InputReader:
                     m = abs(m)
                 RBC = numbers[2]
                 # RBC*cos(m*t)*cos(n*p)
-                m_idx = np.where(bdry[:, 0] == m)[0]
-                n_idx = np.where(bdry[:, 1] == n)[0]
+                m_idx = np.where(inputs["surface"][:, 0] == m)[0]
+                n_idx = np.where(inputs["surface"][:, 1] == n)[0]
                 idx = np.where(np.isin(m_idx, n_idx))[0]
                 if np.size(idx):
-                    bdry[m_idx[idx[0]], 2] += RBC
+                    inputs["surface"][m_idx[idx[0]], 2] += RBC
                 else:
-                    bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                    bdry[-1, :] = np.array([m, n, RBC, 0.0])
+                    inputs["surface"] = np.pad(
+                        inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                    )
+                    inputs["surface"][-1, :] = np.array([m, n, RBC, 0.0])
                 if m != 0 and n != 0:  # RBC*sin(m*t)*sin(n*p)
-                    m_idx = np.where(bdry[:, 0] == -m)[0]
-                    n_idx = np.where(bdry[:, 1] == -n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == -m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == -n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 2] += m_sgn * n_sgn * RBC
+                        inputs["surface"][m_idx[idx[0]], 2] += m_sgn * n_sgn * RBC
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([-m, -n, m_sgn * n_sgn * RBC, 0.0])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array(
+                            [-m, -n, m_sgn * n_sgn * RBC, 0.0]
+                        )
             # ZBS*sin(m*t-n*p) = ZBS*sin(m*t)*cos(n*p) - ZBS*cos(m*t)*sin(n*p)
             match = re.search(
                 r"ZBS\(\s*"
                 + num_form
-                + "\s*,\s*"
+                + r"\s*,\s*"
                 + num_form
-                + "\s*\)\s*=\s*"
+                + r"\s*\)\s*=\s*"
                 + num_form,
                 command,
                 re.IGNORECASE,
@@ -966,30 +1068,34 @@ class InputReader:
                     m = abs(m)
                 ZBS = numbers[2]
                 if m != 0:  # ZBS*sin(m*t)*cos(n*p)
-                    m_idx = np.where(bdry[:, 0] == -m)[0]
-                    n_idx = np.where(bdry[:, 1] == n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == -m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 3] += m_sgn * ZBS
+                        inputs["surface"][m_idx[idx[0]], 3] += m_sgn * ZBS
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([-m, n, 0.0, m_sgn * ZBS])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array([-m, n, 0.0, m_sgn * ZBS])
                 if n != 0:  # -ZBS*cos(m*t)*sin(n*p)
-                    m_idx = np.where(bdry[:, 0] == m)[0]
-                    n_idx = np.where(bdry[:, 1] == -n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == -n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 3] += -n_sgn * ZBS
+                        inputs["surface"][m_idx[idx[0]], 3] += -n_sgn * ZBS
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([m, -n, 0.0, -n_sgn * ZBS])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array([m, -n, 0.0, -n_sgn * ZBS])
             # ZBC*cos(m*t-n*p) = ZBC*cos(m*t)*cos(n*p) + ZBC*sin(m*t)*sin(n*p)
             match = re.search(
                 r"ZBC\(\s*"
                 + num_form
-                + "\s*,\s*"
+                + r"\s*,\s*"
                 + num_form
-                + "\s*\)\s*=\s*"
+                + r"\s*\)\s*=\s*"
                 + num_form,
                 command,
                 re.IGNORECASE,
@@ -1010,23 +1116,29 @@ class InputReader:
                     m = abs(m)
                 ZBC = numbers[2]
                 # ZBC*cos(m*t)*cos(n*p)
-                m_idx = np.where(bdry[:, 0] == m)[0]
-                n_idx = np.where(bdry[:, 1] == n)[0]
+                m_idx = np.where(inputs["surface"][:, 0] == m)[0]
+                n_idx = np.where(inputs["surface"][:, 1] == n)[0]
                 idx = np.where(np.isin(m_idx, n_idx))[0]
                 if np.size(idx):
-                    bdry[m_idx[idx[0]], 3] += ZBC
+                    inputs["surface"][m_idx[idx[0]], 3] += ZBC
                 else:
-                    bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                    bdry[-1, :] = np.array([m, n, 0.0, ZBC])
+                    inputs["surface"] = np.pad(
+                        inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                    )
+                    inputs["surface"][-1, :] = np.array([m, n, 0.0, ZBC])
                 if m != 0 and n != 0:  # ZBC*sin(m*t)*sin(n*p)
-                    m_idx = np.where(bdry[:, 0] == -m)[0]
-                    n_idx = np.where(bdry[:, 1] == -n)[0]
+                    m_idx = np.where(inputs["surface"][:, 0] == -m)[0]
+                    n_idx = np.where(inputs["surface"][:, 1] == -n)[0]
                     idx = np.where(np.isin(m_idx, n_idx))[0]
                     if np.size(idx):
-                        bdry[m_idx[idx[0]], 3] += m_sgn * n_sgn * ZBC
+                        inputs["surface"][m_idx[idx[0]], 3] += m_sgn * n_sgn * ZBC
                     else:
-                        bdry = np.pad(bdry, ((0, 1), (0, 0)), mode="constant")
-                        bdry[-1, :] = np.array([-m, -n, 0.0, m_sgn * n_sgn * ZBC])
+                        inputs["surface"] = np.pad(
+                            inputs["surface"], ((0, 1), (0, 0)), mode="constant"
+                        )
+                        inputs["surface"][-1, :] = np.array(
+                            [-m, -n, 0.0, m_sgn * n_sgn * ZBC]
+                        )
 
             # catch multi-line inputs
             match = re.search(r"=", command)
@@ -1040,47 +1152,11 @@ class InputReader:
                     raise IOError(
                         colored("Cannot handle multi-line VMEC inputs!", "red")
                     )
-
-        p_l *= pres_scale
-        desc_file.write("\n")
-        desc_file.write("# pressure and rotational transform profiles\n")
-        for k in range(max(p_l.size, i_l.size)):
-            if k >= p_l.size:
-                desc_file.write(
-                    "l: {:3d}  p = {:16.8E}  i = {:16.8E}\n".format(k, 0.0, i_l[k])
-                )
-            elif k >= i_l.size:
-                desc_file.write(
-                    "l: {:3d}  p = {:16.8E}  i = {:16.8E}\n".format(k, p_l[k], 0.0)
-                )
-            else:
-                desc_file.write(
-                    "l: {:3d}  p = {:16.8E}  i = {:16.8E}\n".format(k, p_l[k], i_l[k])
-                )
-
-        desc_file.write("\n")
-        desc_file.write("# magnetic axis initial guess\n")
-        for k in range(np.shape(axis)[0]):
-            desc_file.write(
-                "n: {:3d}  R0 = {:16.8E}  Z0 = {:16.8E}\n".format(
-                    int(axis[k, 0]), axis[k, 1], axis[k, 2]
-                )
-            )
-
-        desc_file.write("\n")
-        desc_file.write("# fixed-boundary surface shape\n")
-        for k in range(np.shape(bdry)[0]):
-            desc_file.write(
-                "m: {:3d}  n: {:3d}  R1 = {:16.8E}  Z1 = {:16.8E}\n".format(
-                    int(bdry[k, 0]), int(bdry[k, 1]), bdry[k, 2], bdry[k, 3]
-                )
-            )
-
-        desc_file.truncate()
-
-        # close files
+        inputs["surface"] = np.pad(inputs["surface"], ((0, 0), (1, 0)), mode="constant")
+        inputs["pressure"][:, 1] *= pres_scale
         vmec_file.close()
-        desc_file.close()
+
+        return inputs
 
 
 # NOTE: this has to be outside the class to work with autodoc
@@ -1120,10 +1196,10 @@ def get_parser():
         + "and three times (-ppp) to show all iterations.",
     )
     parser.add_argument(
-        "--vmec",
-        metavar="vmec_path",
+        "--guess",
+        metavar="path",
         default=None,
-        help="Path to VMEC data for initial guess.",
+        help="Path to DESC or VMEC equilibrium for initial guess.",
     )
     parser.add_argument(
         "--gpu",
