@@ -1,13 +1,12 @@
 import numpy as np
-from termcolor import colored
 from abc import ABC, abstractmethod
 import warnings
 import scipy.optimize
 
-from desc.backend import jnp, put
+from desc.backend import jnp, jit
 from desc.io import IOAble
-from desc.grid import Grid, LinearGrid, ConcentricGrid, QuadratureGrid
-from desc.interpolate import interp1d
+from desc.grid import Grid
+from desc.interpolate import interp1d, _approx_df
 from desc.transform import Transform
 from desc.basis import PowerSeries
 from desc.utils import copy_coeffs
@@ -93,6 +92,8 @@ class PowerSeriesProfile(Profile):
         mode numbers for the associated coefficients. eg a[modes[i]] = params[i]
     grid : Grid
         default grid to use for computing values using transform method
+    sym : bool
+        Whether the basis should only contain even powers (True) or all powers (False).
     name : str
         name of the profile
 
@@ -100,13 +101,31 @@ class PowerSeriesProfile(Profile):
 
     _io_attrs_ = Profile._io_attrs_ + ["_basis", "_transform"]
 
-    def __init__(self, params, modes=None, grid=None, name=None):
+    def __init__(self, params, modes=None, grid=None, sym="auto", name=""):
 
         self._name = name
         params = np.atleast_1d(params)
+
+        if (
+            sym == "auto"
+        ):  # check if all odd terms are zero, if so return even. Print something when does so?
+            if modes is None:
+                modes = np.arange(params.size)
+            else:
+                modes = np.atleast_1d(modes)
+            if np.all(params[modes % 2 != 0] == 0):
+                sym = "even"
+            else:
+                sym = False
+        self.sym = "even" if sym else False
         if modes is None:
-            modes = np.arange(params.size)
-        self._basis = PowerSeries(L=int(np.max(abs(modes))))
+            if sym:
+                modes = np.arange(2 * params.size, step=2)
+            else:
+                modes = np.arange(params.size)
+        else:
+            modes = np.atleast_1d(modes)
+        self._basis = PowerSeries(L=int(np.max(abs(modes))), sym=self.sym)
         self._params = np.zeros(self.basis.num_modes, dtype=float)
         for m, c in zip(modes, params):
             idx = np.where(self.basis.modes[:, 0] == int(m))[0]
@@ -130,11 +149,12 @@ class PowerSeriesProfile(Profile):
             grid,
             self.basis,
             derivs=np.array([[0, 0, 0], [1, 0, 0], [2, 0, 0], [3, 0, 0]]),
+            build=True,
         )
         return transform
 
     def __repr__(self):
-        s = super().__repr__
+        s = super().__repr__()
         s = s[:-1]
         s += ", basis={})".format(self.basis)
         return s
@@ -160,6 +180,7 @@ class PowerSeriesProfile(Profile):
                 f"grid should be a Grid or subclass, or ndarray, got {type(new)}"
             )
         self._transform.grid = self.grid
+        self._transform.build()
 
     @property
     def params(self):
@@ -230,7 +251,7 @@ class PowerSeriesProfile(Profile):
         return transform.transform(params, dr=dr, dt=dt, dz=dz)
 
     @classmethod
-    def from_values(cls, x, y, order=6, rcond=None, w=None, grid=None, name=None):
+    def from_values(cls, x, y, order=6, rcond=None, w=None, grid=None, name=""):
         """Fit a PowerSeriesProfile from point data
 
         Parameters
@@ -395,23 +416,29 @@ class SplineProfile(Profile):
 
     """
 
-    _io_attrs_ = Profile._io_attrs_ + ["_knots", "_method"]
+    _io_attrs_ = Profile._io_attrs_ + ["_knots", "_method", "_Dx"]
 
-    def __init__(self, values, knots=None, grid=None, method="cubic2", name=None):
+    def __init__(self, values, knots=None, grid=None, method="cubic2", name=""):
 
+        values = np.atleast_1d(values)
         if knots is None:
             knots = np.linspace(0, 1, values.size)
+        else:
+            knots = np.atleast_1d(knots)
         self._name = name
         self._knots = knots
         self._params = values
         self._method = method
+        self._Dx = _approx_df(
+            self._knots, np.eye(self._knots.size), self._method, axis=0
+        )
 
         if grid is None:
             grid = Grid(np.empty((0, 3)))
         self.grid = grid
 
     def __repr__(self):
-        s = super().__repr__
+        s = super().__repr__()
         s = s[:-1]
         s += ", method={}, num_knots={})".format(self._method, len(self._knots))
         return s
@@ -498,7 +525,8 @@ class SplineProfile(Profile):
             return jnp.zeros_like(xq)
         x = self._knots
         f = params
-        fq = interp1d(xq, x, f, method=self._method, derivative=dr, extrap=True)
+        df = self._Dx @ f
+        fq = interp1d(xq, x, f, method=self._method, derivative=dr, extrap=True, df=df)
         return fq
 
     def to_powerseries(self, order=6, xs=100, rcond=None, w=None):
@@ -630,7 +658,7 @@ class MTanhProfile(Profile):
 
     """
 
-    def __init__(self, params, grid=None, name=None):
+    def __init__(self, params, grid=None, name=""):
 
         self._name = name
         self._params = params
@@ -640,7 +668,7 @@ class MTanhProfile(Profile):
         self.grid = grid
 
     def __repr__(self):
-        s = super().__repr__
+        s = super().__repr__()
         s = s[:-1]
         s += ", num_params={})".format(len(self._params))
         return s
@@ -727,7 +755,7 @@ class MTanhProfile(Profile):
                 / (width ** 3 * (e2z + 1) ** 3)
             )
             f = (
-                jnp.polyval(jnp.polyder(core_poly[::-1], 2)) * dz ** 2
+                jnp.polyval(jnp.polyder(core_poly[::-1], 2), zz) * dz ** 2
                 + jnp.polyval(jnp.polyder(core_poly[::-1], 1), zz) * ddz
             )
 
@@ -792,7 +820,7 @@ class MTanhProfile(Profile):
         pmax=None,
         pmin=None,
         grid=None,
-        name=None,
+        name="",
         **kwargs,
     ):
         """Fit a MTanhProfile from point data
@@ -838,7 +866,7 @@ class MTanhProfile(Profile):
             )
             / w
         )
-
+        fun = jit(fun)
         ped0 = np.clip(interp1d([0.93], x, y, "cubic2", extrap=True), 0, np.inf)[0]
         off0 = np.clip(interp1d([0.98], x, y, "cubic2", extrap=True), 0, np.inf)[0]
         default_pmax = np.array([np.inf, np.inf, 1.02, 0.2, np.inf])
