@@ -46,7 +46,7 @@ class Transform(IOAble):
         basis,
         derivs=0,
         rcond="auto",
-        build=False,
+        build=True,
         build_pinv=False,
         method="auto",
     ):
@@ -230,11 +230,11 @@ class Transform(IOAble):
             warnings.warn(
                 colored(
                     "fft method requires nodes to be sorted by toroidal angle in "
-                    + "ascending order, falling back to direct2 method",
+                    + "ascending order, falling back to direct1 method",
                     "yellow",
                 )
             )
-            self.method = "direct2"
+            self.method = "direct1"
             return
 
         if len(zeta_vals) > 1 and not islinspaced(zeta_vals):
@@ -363,7 +363,7 @@ class Transform(IOAble):
         if self.method == "direct1":
             for d in self.derivatives:
                 self._matrices["direct1"][d[0]][d[1]][d[2]] = self.basis.evaluate(
-                    self.grid.nodes, d
+                    self.grid.nodes, d, unique=True
                 )
 
         if self.method in ["fft", "direct2"]:
@@ -373,7 +373,7 @@ class Transform(IOAble):
             temp_modes = np.hstack([self.lm_modes, np.zeros((self.num_lm_modes, 1))])
             for d in temp_d:
                 self.matrices["fft"][d[0]][d[1]] = self.basis.evaluate(
-                    self.fft_nodes, d, modes=temp_modes
+                    self.fft_nodes, d, modes=temp_modes, unique=True
                 )
         if self.method == "direct2":
             temp_d = np.hstack(
@@ -384,7 +384,7 @@ class Transform(IOAble):
             )
             for d in temp_d:
                 self.matrices["direct2"][d[2]] = self.basis.evaluate(
-                    self.dft_nodes, d, modes=temp_modes
+                    self.dft_nodes, d, modes=temp_modes, unique=True
                 )
 
         self._built = True
@@ -423,7 +423,9 @@ class Transform(IOAble):
             array of values of function at node locations
         """
         if not self.built:
-            self.build()
+            raise RuntimeError(
+                "Transform must be precomputed with transform.build() before being used"
+            )
 
         if self.basis.num_modes != c.size:
             raise ValueError(
@@ -507,8 +509,14 @@ class Transform(IOAble):
 
         """
         if not self.built_pinv:
-            self.build_pinv()
-        return jnp.matmul(self.matrices["pinv"], self.grid.weights * x)
+            raise RuntimeError(
+                "Transform must be precomputed with transform.build_pinv() before being used"
+            )
+        if x.ndim > 1:
+            weights = self.grid.weights.reshape((-1, 1))
+        else:
+            weights = self.grid.weights
+        return jnp.matmul(self.matrices["pinv"], weights * x)
 
     def project(self, y):
         """Project vector y onto basis.
@@ -527,7 +535,9 @@ class Transform(IOAble):
             vector y projected onto basis, shape (self.basis.num_modes)
         """
         if not self.built:
-            self.build()
+            raise RuntimeError(
+                "Transform must be precomputed with transform.build() before being used"
+            )
 
         if self.grid.num_nodes != y.size:
             raise ValueError(
@@ -599,9 +609,7 @@ class Transform(IOAble):
     @property
     def grid(self):
         """Grid : collocation grid for the transform."""
-        if not hasattr(self, "_grid"):
-            self._grid = None
-        return self._grid
+        return self.__dict__.setdefault("_grid", None)
 
     @grid.setter
     def grid(self, grid):
@@ -621,9 +629,7 @@ class Transform(IOAble):
     @property
     def basis(self):
         """Basis : spectral basis for the transform."""
-        if not hasattr(self, "_basis"):
-            self._basis = None
-        return self._basis
+        return self.__dict__.setdefault("_basis", None)
 
     @basis.setter
     def basis(self, basis):
@@ -687,16 +693,17 @@ class Transform(IOAble):
     @property
     def matrices(self):
         """dict of ndarray : transform matrices such that x=A*c."""
-        if not hasattr(self, "_matrices"):
-            self._matrices = {
+        return self.__dict__.setdefault(
+            "_matrices",
+            {
                 "direct1": {
                     i: {j: {k: {} for k in range(4)} for j in range(4)}
                     for i in range(4)
                 },
                 "fft": {i: {j: {} for j in range(4)} for i in range(4)},
                 "direct2": {i: {} for i in range(4)},
-            }
-        return self._matrices
+            },
+        )
 
     @property
     def num_nodes(self):
@@ -709,42 +716,34 @@ class Transform(IOAble):
         return self.basis.num_modes
 
     @property
-    def modes(self):
+    def nodes(self):
         """ndarray: collocation nodes."""
         return self.grid.nodes
 
     @property
-    def nodes(self):
+    def modes(self):
         """ndarray: spectral mode numbers."""
-        return self.basis.nodes
+        return self.basis.modes
 
     @property
     def built(self):
         """bool : whether the transform matrices have been built."""
-        if not hasattr(self, "_built"):
-            self._built = False
-        return self._built
+        return self.__dict__.setdefault("_built", False)
 
     @property
     def built_pinv(self):
         """bool : whether the pseudoinverse matrix has been built."""
-        if not hasattr(self, "_built_pinv"):
-            self._built_pinv = False
-        return self._built_pinv
+        return self.__dict__.setdefault("_built_pinv", False)
 
     @property
     def rcond(self):
         """float: reciprocal condition number for inverse transform."""
-        if not hasattr(self, "_rcond"):
-            self._rcond = "auto"
-        return self._rcond
+        return self.__dict__.setdefault("_rcond", "auto")
 
     @property
     def method(self):
         """{``'direct1'``, ``'direct2'``, ``'fft'``}: method of computing transform."""
-        if not hasattr(self, "_method"):
-            self._method = "direct1"
-        return self._method
+        return self.__dict__.setdefault("_method", "direct1")
 
     @method.setter
     def method(self, method):
