@@ -1,8 +1,10 @@
 import numpy as np
 import warnings
 from termcolor import colored
+from shapely.geometry import Polygon
 
-# Helper Classes -------------------------------------------------------------
+
+# Helper Classes -----------------------------------------------------------------------
 
 
 class Timer:
@@ -169,7 +171,7 @@ class _Indexable:
 
     This is a singleton object that overrides the ``__getitem__`` method
     to return the index it is passed.
-    >>> opsindex[1:2, 3, None, ..., ::2]
+    >>> Index[1:2, 3, None, ..., ::2]
     (slice(1, 2, None), 3, None, Ellipsis, slice(None, None, 2))
     copied from jax.ops.index to work with either backend
     """
@@ -184,11 +186,11 @@ class _Indexable:
 Helper object for building indexes for indexed update functions.
 This is a singleton object that overrides the ``__getitem__`` method
 to return the index it is passed.
->>> opsindex[1:2, 3, None, ..., ::2]
+>>> Index[1:2, 3, None, ..., ::2]
 (slice(1, 2, None), 3, None, Ellipsis, slice(None, None, 2))
 copied from jax.ops.index to work with either backend
 """
-opsindex = _Indexable()
+Index = _Indexable()
 
 
 # Helper Functions -----------------------------------------------------------
@@ -196,7 +198,6 @@ opsindex = _Indexable()
 
 def unpack_state(x, nR, nZ):
     """Unpack the state vector x into R_lmn, Z_lmn, L_lmn components.
-
     Parameters
     ----------
     x : ndarray
@@ -205,7 +206,6 @@ def unpack_state(x, nR, nZ):
         number of R_lmn coefficients
     nZ : int
         number of Z_lmn coefficients
-
     Returns
     -------
     R_lmn : ndarray
@@ -214,7 +214,6 @@ def unpack_state(x, nR, nZ):
         spectral coefficients of Z
     L_lmn : ndarray
         spectral coefficients of lambda
-
     """
     R_lmn = x[:nR]
     Z_lmn = x[nR : nR + nZ]
@@ -222,8 +221,74 @@ def unpack_state(x, nR, nZ):
     return R_lmn, Z_lmn, L_lmn
 
 
+def area_difference(Rr1, Rr2, Zr1, Zr2, Rv1, Rv2, Zv1, Zv2):
+    """Compute area difference between coordinate curves
+
+    Parameters
+    ----------
+    args : ndarray
+        R and Z coordinates of constant rho (r) or vartheta (v) contours.
+        Arrays should be indexed as [rho,theta,zeta]
+
+    Returns
+    -------
+    area_rho : ndarray, shape(Nr, Nz)
+        normalized area difference of rho contours, computed as the symmetric
+        difference divided by the intersection
+    area_theta : ndarray, shape(Nt, Nz)
+        normalized area difference between vartheta contours, computed as the area
+        of the polygon created by closing the two vartheta contours divided by the
+        perimeter squared
+    """
+    assert Rr1.shape == Rr2.shape == Zr1.shape == Zr2.shape
+    assert Rv1.shape == Rv2.shape == Zv1.shape == Zv2.shape
+
+    poly_r1 = np.array(
+        [
+            [Polygon(np.array([R, Z]).T) for R, Z in zip(Rr1[:, :, i], Zr1[:, :, i])]
+            for i in range(Rr1.shape[2])
+        ]
+    )
+    poly_r2 = np.array(
+        [
+            [Polygon(np.array([R, Z]).T) for R, Z in zip(Rr2[:, :, i], Zr2[:, :, i])]
+            for i in range(Rr2.shape[2])
+        ]
+    )
+    poly_v = np.array(
+        [
+            [
+                Polygon(np.array([R, Z]).T)
+                for R, Z in zip(
+                    np.hstack([Rv1[:, :, i].T, Rv2[::-1, :, i].T]),
+                    np.hstack([Zv1[:, :, i].T, Zv2[::-1, :, i].T]),
+                )
+            ]
+            for i in range(Rv1.shape[2])
+        ]
+    )
+
+    diff_rho = np.array(
+        [
+            poly1.symmetric_difference(poly2).area
+            for poly1, poly2 in zip(poly_r1.flat, poly_r2.flat)
+        ]
+    ).reshape((Rr1.shape[2], Rr1.shape[0]))
+    intersect_rho = np.array(
+        [
+            poly1.intersection(poly2).area
+            for poly1, poly2 in zip(poly_r1.flat, poly_r2.flat)
+        ]
+    ).reshape((Rr1.shape[2], Rr1.shape[0]))
+    area_rho = np.where(diff_rho > 0, diff_rho / intersect_rho, 0)
+    area_theta = np.array(
+        [poly.area / (poly.length) ** 2 for poly in poly_v.flat]
+    ).reshape((Rv1.shape[1], Rv1.shape[2]))
+    return area_rho, area_theta
+
+
 def equals(a, b):
-    """Compare (possibly nested) objects, such as dicts and lists
+    """Compare (possibly nested) objects, such as dicts and lists.
 
     Parameters
     ----------
@@ -354,28 +419,8 @@ def islinspaced(x, axis=-1, tol=1e-12):
     return np.all(np.diff(x, axis=axis).std() < tol)
 
 
-def sign(x):
-    """Sign function, but returns 1 for x==0.
-
-    Parameters
-    ----------
-    x : array-like
-        array of input values
-
-    Returns
-    -------
-    y : array-like
-        1 where x>=0, -1 where x<0
-
-    """
-    x = np.atleast_1d(x)
-    y = np.where(x == 0, 1, np.sign(x))
-    return y
-
-
 def copy_coeffs(c_old, modes_old, modes_new, c_new=None):
     """Copy coefficients from one resolution to another."""
-
     modes_old, modes_new = np.atleast_1d(modes_old), np.atleast_1d(modes_new)
     if modes_old.ndim == 1:
         modes_old = modes_old.reshape((-1, 1))
@@ -393,12 +438,33 @@ def copy_coeffs(c_old, modes_old, modes_new, c_new=None):
     return c_new
 
 
-def expand_state(x, old_R, new_R, old_Z, new_Z, old_L, new_L):
-    """Copy a state vector from one resolution to another."""
-    R_lmn, Z_lmn, L_lmn = unpack_state(x, len(old_R), len(old_Z))
-    R_lmn = copy_coeffs(R_lmn, old_R, new_R)
-    Z_lmn = copy_coeffs(Z_lmn, old_Z, new_Z)
-    L_lmn = copy_coeffs(L_lmn, old_L, new_L)
+def svd_inv_null(A):
+    """Compute pseudo-inverse and null space of a matrix using an SVD.
 
-    x = np.concatenate([R_lmn, Z_lmn, L_lmn])
-    return x
+    Parameters
+    ----------
+    A : ndarray
+        Matrix to invert and find null space of.
+
+    Returns
+    -------
+    Ainv : ndarray
+        Pseudo-inverse of A.
+    Z : ndarray
+        Null space of A.
+
+    """
+    u, s, vh = np.linalg.svd(A, full_matrices=True)
+    M, N = u.shape[0], vh.shape[1]
+    K = min(M, N)
+    rcond = np.finfo(A.dtype).eps * max(M, N)
+    tol = np.amax(s) * rcond
+    large = s > tol
+    num = np.sum(large, dtype=int)
+    uk = u[:, :K]
+    vhk = vh[:K, :]
+    s = np.divide(1, s, where=large, out=s)
+    s[(~large,)] = 0
+    Ainv = np.matmul(vhk.T, np.multiply(s[..., np.newaxis], uk.T))
+    Z = vh[num:, :].T.conj()
+    return Ainv, Z
