@@ -1,18 +1,23 @@
 import os
+import warnings
 import numpy as np
-
-from netCDF4 import Dataset, stringtochar
-from shapely.geometry import Polygon
 import matplotlib.pyplot as plt
+from netCDF4 import Dataset, stringtochar
 from scipy import optimize, interpolate, integrate
 
-from desc.utils import Timer, sign
+from desc.backend import sign
+from desc.utils import Timer, area_difference
 from desc.grid import Grid, LinearGrid
 from desc.basis import DoubleFourierSeries
 from desc.transform import Transform
 from desc.profiles import PowerSeriesProfile
 from desc.equilibrium import Equilibrium
-from desc.boundary_conditions import LCFSConstraint
+from desc.objectives import (
+    ObjectiveFunction,
+    FixBoundaryR,
+    FixBoundaryZ,
+)
+from desc.objectives.utils import factorize_linear_constraints
 from desc.vmec_utils import (
     ptolemy_identity_fwd,
     ptolemy_identity_rev,
@@ -25,7 +30,7 @@ class VMECIO:
     """Performs input from VMEC netCDF files to DESC Equilibrium and vice-versa."""
 
     @classmethod
-    def load(cls, path, L=-1, M=-1, N=-1, spectral_indexing="ansi"):
+    def load(cls, path, L=-1, M=-1, N=-1, spectral_indexing="fringe"):
         """Load a VMEC netCDF file as a Equilibrium.
 
         Parameters
@@ -39,7 +44,7 @@ class VMECIO:
         N : int, optional
             Toroidal resolution. Default = NTOR from VMEC solution.
         spectral_indexing : str, optional
-            Type of Zernike indexing scheme to use. (Default = ``'ansi'``)
+            Type of Zernike indexing scheme to use. (Default = ``'fringe'``)
 
         Returns
         -------
@@ -50,6 +55,14 @@ class VMECIO:
         file = Dataset(path, mode="r")
         inputs = {}
 
+        version = file.variables["version_"][0]
+        if version < 9:
+            warnings.warn(
+                "VMEC output appears to be from version {}, while DESC is only designed for compatibility with VMEC version 9. Some data may not be loaded correctly.".format(
+                    str(version)
+                )
+            )
+
         # parameters
         inputs["Psi"] = float(file.variables["phi"][-1])
         inputs["NFP"] = int(file.variables["nfp"][0])
@@ -59,8 +72,6 @@ class VMECIO:
         default_L = {
             "ansi": inputs["M"],
             "fringe": 2 * inputs["M"],
-            "chevron": inputs["M"],
-            "house": 2 * inputs["M"],
         }
         inputs["L"] = L if L >= 0 else default_L[inputs["spectral_indexing"]]
 
@@ -84,20 +95,22 @@ class VMECIO:
         # profiles
         preset = file.dimensions["preset"].size
         p0 = file.variables["presf"][0] / file.variables["am"][0]
-        inputs["profiles"] = np.zeros((preset, 3))
-        inputs["profiles"][:, 0] = np.arange(0, 2 * preset, 2)
-        inputs["profiles"][:, 1] = file.variables["am"][:] * p0
-        inputs["profiles"][:, 2] = file.variables["ai"][:]
+        inputs["pressure"] = np.zeros((preset, 2))
+        inputs["pressure"][:, 0] = np.arange(0, 2 * preset, 2)
+        inputs["pressure"][:, 1] = file.variables["am"][:] * p0
+        inputs["iota"] = np.zeros((preset, 2))
+        inputs["iota"][:, 0] = np.arange(0, 2 * preset, 2)
+        inputs["iota"][:, 1] = file.variables["ai"][:]
 
         file.close
 
         # boundary
         m, n, Rb_lmn = ptolemy_identity_fwd(xm, xn, s=rmns[-1, :], c=rmnc[-1, :])
         m, n, Zb_lmn = ptolemy_identity_fwd(xm, xn, s=zmns[-1, :], c=zmnc[-1, :])
-        inputs["boundary"] = np.vstack((np.zeros_like(m), m, n, Rb_lmn, Zb_lmn)).T
+        inputs["surface"] = np.vstack((np.zeros_like(m), m, n, Rb_lmn, Zb_lmn)).T
 
         # initialize Equilibrium
-        eq = Equilibrium(inputs=inputs)
+        eq = Equilibrium(**inputs)
 
         # R
         m, n, R_mn = ptolemy_identity_fwd(xm, xn, s=rmns, c=rmnc)
@@ -112,12 +125,14 @@ class VMECIO:
         eq.L_lmn = fourier_to_zernike(m, n, L_mn, eq.L_basis)
 
         # apply boundary conditions
-        BC = eq.surface.get_constraint(
-            eq.R_basis,
-            eq.Z_basis,
-            eq.L_basis,
+        constraints = (FixBoundaryR(), FixBoundaryZ())
+        objective = ObjectiveFunction(constraints, eq=eq, verbose=0)
+        xp, A, Ainv, b, Z, unfixed_idx, project, recover = factorize_linear_constraints(
+            constraints, extra_args=objective.args
         )
-        eq.x = BC.make_feasible(eq.x)
+        args = objective.unpack_state(recover(project(objective.x(eq))))
+        for key, value in args.items():
+            setattr(eq, key, value)
 
         return eq
 
@@ -276,7 +291,7 @@ class VMECIO:
 
         signgs = file.createVariable("signgs", np.int32)
         signgs.long_name = "sign of coordinate system jacobian"
-        signgs[:] = sign(eq.compute_jacobian(Grid(np.array([[1, 0, 0]])))["g"])
+        signgs[:] = sign(eq.compute("sqrt(g)", Grid(np.array([[1, 0, 0]])))["sqrt(g)"])
 
         gamma = file.createVariable("gamma", np.float64)
         gamma.long_name = "compressibility index (0 = pressure prescribed)"
@@ -307,14 +322,16 @@ class VMECIO:
         am[:] = np.zeros((file.dimensions["preset"].size,))
         # only using up to 10th order to avoid poor conditioning
         am[:11] = PowerSeriesProfile.from_values(
-            s_full, eq.pressure(r_full), order=10).params
+            s_full, eq.pressure(r_full), order=10
+        ).params
 
         ai = file.createVariable("ai", np.float64, ("preset",))
         ai.long_name = "rotational transform coefficients"
         ai[:] = np.zeros((file.dimensions["preset"].size,))
         # only using up to 10th order to avoid poor conditioning
         ai[:11] = PowerSeriesProfile.from_values(
-            s_full, eq.iota(r_full), order=10).params
+            s_full, eq.iota(r_full), order=10
+        ).params
 
         ac = file.createVariable("ac", np.float64, ("preset",))
         ac.long_name = "normalized toroidal current density coefficients"
@@ -362,7 +379,12 @@ class VMECIO:
         chi = file.createVariable("chi", np.float64, ("radius",))
         chi.long_name = "poloidal flux"
         chi.units = "Wb"
-        chi[:] = 2 * Psi * integrate.cumtrapz(r_full * iotaf[:], r_full, initial=0)
+        chi[:] = (
+            2
+            * Psi
+            * signgs[:]
+            * integrate.cumtrapz(r_full * iotaf[:], r_full, initial=0)
+        )
 
         chipf = file.createVariable("chipf", np.float64, ("radius",))
         chipf.long_name = "d(chi)/ds: poloidal flux derivative"
@@ -371,21 +393,21 @@ class VMECIO:
         Rmajor_p = file.createVariable("Rmajor_p", np.float64)
         Rmajor_p.long_name = "major radius"
         Rmajor_p.units = "m"
-        Rmajor_p[:] = eq.major_radius
+        Rmajor_p[:] = eq.compute("R0")["R0"]
 
         Aminor_p = file.createVariable("Aminor_p", np.float64)
         Aminor_p.long_name = "minor radius"
         Aminor_p.units = "m"
-        Aminor_p[:] = eq.minor_radius
+        Aminor_p[:] = eq.compute("a")["a"]
 
         aspect = file.createVariable("aspect", np.float64)
         aspect.long_name = "aspect ratio = R_major / A_minor"
-        aspect[:] = eq.aspect_ratio
+        aspect[:] = eq.compute("R0/a")["R0/a"]
 
         volume_p = file.createVariable("volume_p", np.float64)
         volume_p.long_name = "plasma volume"
         volume_p.units = "m^3"
-        volume_p[:] = eq.compute_volume()
+        volume_p[:] = eq.compute("V")["V"]
 
         timer.stop("parameters")
         if verbose > 1:
@@ -486,15 +508,21 @@ class VMECIO:
         # derived quantities (approximate conversion)
 
         grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=1)
-        coords = eq.compute_toroidal_coords(grid)
+        coords = eq.compute("R", grid)
         if eq.sym:
             sin_basis = DoubleFourierSeries(M=M_nyq, N=N_nyq, NFP=NFP, sym="sin")
             cos_basis = DoubleFourierSeries(M=M_nyq, N=N_nyq, NFP=NFP, sym="cos")
-            sin_transform = Transform(grid=grid, basis=sin_basis, build_pinv=True)
-            cos_transform = Transform(grid=grid, basis=cos_basis, build_pinv=True)
+            sin_transform = Transform(
+                grid=grid, basis=sin_basis, build=False, build_pinv=True
+            )
+            cos_transform = Transform(
+                grid=grid, basis=cos_basis, build=False, build_pinv=True
+            )
         else:
             full_basis = DoubleFourierSeries(M=M_nyq, N=N_nyq, NFP=NFP, sym=None)
-            full_transform = Transform(grid=grid, basis=full_basis, build_pinv=True)
+            full_transform = Transform(
+                grid=grid, basis=full_basis, build=False, build_pinv=True
+            )
 
         rmin_surf = file.createVariable("rmin_surf", np.float64)
         rmin_surf.long_name = "minimum R coordinate range"
@@ -511,6 +539,10 @@ class VMECIO:
         zmax_surf.units = "m"
         zmax_surf[:] = np.amax(np.abs(coords["Z"]))
 
+        # half grid quantities
+        half_grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half)
+        data_half_grid = eq.compute("|B|", half_grid)
+        data_half_grid = eq.compute("J", half_grid, data=data_half_grid)
         # Jacobian
         timer.start("Jacobian")
         if verbose > 0:
@@ -527,13 +559,16 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_jacobian(grid)["g"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["sqrt(g)"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         gmnc[0, :] = 0
         gmnc[1:, :] = c
@@ -560,13 +595,16 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_magnetic_field(grid)["|B|"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["|B|"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bmnc[0, :] = 0
         bmnc[1:, :] = c
@@ -597,19 +635,22 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_magnetic_field(grid)["B^theta"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["B^theta"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bsupumnc[0, :] = 0
-        bsupumnc[1:, :] = c
+        bsupumnc[1:, :] = c * signgs[:]
         if not eq.sym:
             bsupumns[0, :] = 0
-            bsupumns[1:, :] = s
+            bsupumns[1:, :] = s * signgs[:]
         timer.stop("B^theta")
         if verbose > 1:
             timer.disp("B^theta")
@@ -634,23 +675,30 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_magnetic_field(grid)["B^zeta"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["B^zeta"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bsupvmnc[0, :] = 0
-        bsupvmnc[1:, :] = c
+        bsupvmnc[1:, :] = c * signgs[:]
         if not eq.sym:
             bsupvmns[0, :] = 0
-            bsupvmns[1:, :] = s
+            bsupvmns[1:, :] = s * signgs[:]
         timer.stop("B^zeta")
         if verbose > 1:
             timer.disp("B^zeta")
 
+        # full grid quantities
+        full_grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_full)
+        data_full_grid = eq.compute("J", full_grid)
+
         # B_psi
         timer.start("B_psi")
         if verbose > 0:
@@ -671,14 +719,14 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs, m.size))
-        for k in range(surfs):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_full[k])
-            data = eq.compute_magnetic_field(grid)["B_rho"] / (2 * r_full[k])
-            # B_rho -> B_psi conversion: d(rho)/d(s) = 1/(2*rho)
-            if eq.sym:
-                x_mn[k, :] = sin_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = data_full_grid["B_rho"].reshape(
+            full_grid.M, full_grid.L, full_grid.N, order="F"
+        ).transpose((1, 0, 2)).reshape((full_grid.L, -1)) / (2 * r_full[:, np.newaxis])
+        # B_rho -> B_psi conversion: d(rho)/d(s) = 1/(2*rho)
+        if eq.sym:
+            x_mn[:, :] = sin_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bsubsmns[:, :] = s
         bsubsmns[0, :] = (  # linear extrapolation for coefficient at the magnetic axis
@@ -714,19 +762,22 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_magnetic_field(grid)["B_theta"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["B_theta"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bsubumnc[0, :] = 0
-        bsubumnc[1:, :] = c
+        bsubumnc[1:, :] = c * signgs[:]
         if not eq.sym:
             bsubumns[0, :] = 0
-            bsubumns[1:, :] = s
+            bsubumns[1:, :] = s * signgs[:]
         timer.stop("B_theta")
         if verbose > 1:
             timer.disp("B_theta")
@@ -751,19 +802,22 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs - 1, m.size))
-        for k in range(surfs - 1):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_half[k])
-            data = eq.compute_magnetic_field(grid)["B_zeta"]
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            data_half_grid["B_zeta"]
+            .reshape(half_grid.M, half_grid.L, half_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((half_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         bsubvmnc[0, :] = 0
-        bsubvmnc[1:, :] = c
+        bsubvmnc[1:, :] = c * signgs[:]
         if not eq.sym:
             bsubvmns[0, :] = 0
-            bsubvmns[1:, :] = s
+            bsubvmns[1:, :] = s * signgs[:]
         timer.stop("B_zeta")
         if verbose > 1:
             timer.disp("B_zeta")
@@ -790,16 +844,16 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs, m.size))
-        for k in range(surfs):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_full[k])
-            data = (
-                eq.compute_current_density(grid)["J^theta"]
-                * eq.compute_jacobian(grid)["g"]
-            )
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            (data_full_grid["J^theta"] * data_full_grid["sqrt(g)"])
+            .reshape(full_grid.M, full_grid.L, full_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((full_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         currumnc[:, :] = c
         currumnc[0, :] = (  # linear extrapolation for coefficient at the magnetic axis
@@ -837,16 +891,16 @@ class VMECIO:
             m = full_basis.modes[:, 1]
             n = full_basis.modes[:, 2]
         x_mn = np.zeros((surfs, m.size))
-        for k in range(surfs):
-            grid = LinearGrid(M=2 * M_nyq + 1, N=2 * N_nyq + 1, NFP=NFP, rho=r_full[k])
-            data = (
-                eq.compute_current_density(grid)["J^zeta"]
-                * eq.compute_jacobian(grid)["g"]
-            )
-            if eq.sym:
-                x_mn[k, :] = cos_transform.fit(data)
-            else:
-                x_mn[k, :] = full_transform.fit(data)
+        data = (
+            (data_full_grid["J^zeta"] * data_full_grid["sqrt(g)"])
+            .reshape(full_grid.M, full_grid.L, full_grid.N, order="F")
+            .transpose((1, 0, 2))
+            .reshape((full_grid.L, -1))
+        )
+        if eq.sym:
+            x_mn[:, :] = cos_transform.fit(data.T).T
+        else:
+            x_mn[:, :] = full_transform.fit(data.T).T
         xm, xn, s, c = ptolemy_identity_rev(m, n, x_mn)
         currvmnc[:, :] = c
         currvmnc[0, :] = (  # linear extrapolation for coefficient at the magnetic axis
@@ -1039,11 +1093,13 @@ class VMECIO:
 
         Returns
         -------
-        area : float
-            the average normalized area difference between flux surfaces area between
-            flux surfaces is defined as the symmetric difference between the two shapes,
-            and each is normalized to the nominal area of the flux surface, and finally
-            averaged over the total number of flux surfaces being compared
+        area_rho : ndarray, shape(Nr, Nz)
+            normalized area difference of rho contours, computed as the symmetric
+            difference divided by the intersection
+        area_theta : ndarray, shape(Nt, Nz)
+            normalized area difference between vartheta contours, computed as the area
+            of the polygon created by closing the two vartheta contours divided by the
+            perimeter squared
 
         """
         # 1e-3 tolerance seems reasonable for testing, similar to comparison by eye
@@ -1057,33 +1113,16 @@ class VMECIO:
 
         coords = cls.compute_coord_surfaces(equil, vmec_data, Nr, Nt, **kwargs)
 
-        desc_poly = [
-            [
-                Polygon(np.array([R, Z]).T)
-                for R, Z in zip(
-                    coords["Rr_desc"][:, :, i].T, coords["Zr_desc"][:, :, i].T
-                )
-            ]
-            for i in range(Nz)
-        ]
-        vmec_poly = [
-            [
-                Polygon(np.array([R, Z]).T)
-                for R, Z in zip(
-                    coords["Rr_vmec"][:, :, i].T, coords["Zr_vmec"][:, :, i].T
-                )
-            ]
-            for i in range(Nz)
-        ]
-
-        return np.sum(
-            [
-                desc_poly[iz][ir].symmetric_difference(vmec_poly[iz][ir]).area
-                / vmec_poly[iz][ir].area
-                for ir in range(1, Nr)
-                for iz in range(Nz)
-            ]
-        ) / ((Nr - 1) * Nz)
+        Rr1 = coords["Rr_desc"]
+        Zr1 = coords["Zr_desc"]
+        Rv1 = coords["Rv_desc"]
+        Zv1 = coords["Zv_desc"]
+        Rr2 = coords["Rr_vmec"]
+        Zr2 = coords["Zr_vmec"]
+        Rv2 = coords["Rv_vmec"]
+        Zv2 = coords["Zv_vmec"]
+        area_rho, area_theta = area_difference(Rr1, Rr2, Zr1, Zr2, Rv1, Rv2, Zv1, Zv2)
+        return area_rho, area_theta
 
     @classmethod
     def compute_coord_surfaces(cls, equil, vmec_data, Nr=10, Nt=8, **kwargs):
@@ -1139,8 +1178,8 @@ class VMECIO:
 
         # find theta angles corresponding to desired theta* angles
         v_grid = Grid(equil.compute_theta_coords(t_grid.nodes))
-        r_coords_desc = equil.compute_toroidal_coords(r_grid)
-        v_coords_desc = equil.compute_toroidal_coords(v_grid)
+        r_coords_desc = equil.compute("R", r_grid)
+        v_coords_desc = equil.compute("R", v_grid)
 
         # rho contours
         Rr_desc = r_coords_desc["R"].reshape((r_grid.M, r_grid.L, r_grid.N), order="F")
@@ -1200,6 +1239,7 @@ class VMECIO:
             "Rv_vmec": Rv_vmec.reshape((t_grid.M, t_grid.L, t_grid.N), order="F"),
             "Zv_vmec": Zv_vmec.reshape((t_grid.M, t_grid.L, t_grid.N), order="F"),
         }
+        coords = {key: np.swapaxes(val, 0, 1) for key, val in coords.items()}
         return coords
 
     @classmethod
@@ -1238,14 +1278,14 @@ class VMECIO:
         for k in range(len(ax)):
             ax[k].plot(coords["Rr_vmec"][0, 0, k], coords["Zr_vmec"][0, 0, k], "bo")
             s_vmec = ax[k].plot(
-                coords["Rr_vmec"][:, :, k], coords["Zr_vmec"][:, :, k], "b-"
+                coords["Rr_vmec"][:, :, k].T, coords["Zr_vmec"][:, :, k].T, "b-"
             )
-            ax[k].plot(coords["Rv_vmec"][:, :, k].T, coords["Zv_vmec"][:, :, k].T, "b-")
+            ax[k].plot(coords["Rv_vmec"][:, :, k], coords["Zv_vmec"][:, :, k], "b-")
 
-            ax[k].plot(coords["Rr_desc"][0, 0, k], coords["Zr_desc"][0, 0, k], "ro")
-            ax[k].plot(coords["Rv_desc"][:, :, k].T, coords["Zv_desc"][:, :, k].T, "r:")
+            ax[k].plot(coords["Rr_desc"][0, 0, k].T, coords["Zr_desc"][0, 0, k].T, "ro")
+            ax[k].plot(coords["Rv_desc"][:, :, k], coords["Zv_desc"][:, :, k], "r--")
             s_desc = ax[k].plot(
-                coords["Rr_desc"][:, :, k], coords["Zr_desc"][:, :, k], "r:"
+                coords["Rr_desc"][:, :, k].T, coords["Zr_desc"][:, :, k].T, "r--"
             )
 
             ax[k].axis("equal")
